#!/usr/bin/env python
# -- coding: utf-8 --
from ansible.plugins.action import ActionBase
from ansible_collections.cisco.dnac.plugins.action.network_devices_query_count_v1 import ActionModule


class ActionModule2(ActionBase):

    def __init__(self, *args, **kwargs):
        super(ActionModule2, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = True
        self._result = None

    def run(self, tmp=None, task_vars=None):
        module = ActionModule(self._task.args, self._play_context, self._task)
        return module.run(tmp, task_vars)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: security_threats_rogue_allowed_list_v1
short_description: Resource module for Security Threats Rogue Allowed List V1
description:
- Manage operations create and delete of the resource Security Threats Rogue Allowed List V1.
- Intent API to add the threat mac address to allowed list.
- Intent API to remove the threat mac address from allowed list.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  macAddress:
    description: MacAddress path parameter. Threat mac address which needs to be removed
      from the allowed list. Multiple mac addresses will be removed if provided as comma
      separated values (example 00 2A 10 51 22 43,00 2A 10 51 22 44). Note In one request,
      maximum 100 mac addresses can be removed.
    type: str
  payload:
    description: Security Threats Rogue Allowed List's payload.
    elements: dict
    suboptions:
      category:
        description: Category.
        type: int
      macAddress:
        description: Mac Address.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices AddAllowedMacAddressV1
  description: Complete reference of the AddAllowedMacAddressV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-allowed-mac-address
- name: Cisco DNA Center documentation for Devices RemoveAllowedMacAddressV1
  description: Complete reference of the RemoveAllowedMacAddressV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!remove-allowed-mac-address
notes:
  - SDK Method used are
    devices.Devices.add_allowed_mac_address_v1,
    devices.Devices.remove_allowed_mac_address_v1,

  - Paths used are
    post /dna/intent/api/v1/security/threats/rogue/allowed-list,
    delete /dna/intent/api/v1/security/threats/rogue/allowed-list/{macAddress},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.security_threats_rogue_allowed_list_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - category: 0
      macAddress: string

- name: Delete by id
  cisco.dnac.security_threats_rogue_allowed_list_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    macAddress: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": "string",
      "error": {}
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_device_import
short_description: Resource module for Pnp Device Import
description:
- This module represents an alias of the module pnp_device_import_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Pnp Device Import's payload.
    elements: dict
    suboptions:
      _id:
        description: Id.
        type: str
      deviceInfo:
        description: Pnp Device Import's deviceInfo.
        suboptions:
          description:
            description: Description.
            type: str
          deviceSudiSerialNos:
            description: Device Sudi Serial Nos.
            elements: str
            type: list
          hostname:
            description: Hostname.
            type: str
          macAddress:
            description: Mac Address.
            type: str
          pid:
            description: Pid.
            type: str
          serialNumber:
            description: Serial Number.
            type: str
          siteId:
            description: Site Id.
            type: str
          stack:
            description: Stack.
            type: bool
          stackInfo:
            description: Pnp Device Import's stackInfo.
            suboptions:
              isFullRing:
                description: Is Full Ring.
                type: bool
              stackMemberList:
                description: Pnp Device Import's stackMemberList.
                elements: dict
                suboptions:
                  hardwareVersion:
                    description: Hardware Version.
                    type: str
                  licenseLevel:
                    description: License Level.
                    type: str
                  licenseType:
                    description: License Type.
                    type: str
                  macAddress:
                    description: Mac Address.
                    type: str
                  pid:
                    description: Pid.
                    type: str
                  priority:
                    description: Priority.
                    type: float
                  role:
                    description: Role.
                    type: str
                  serialNumber:
                    description: Serial Number.
                    type: str
                  softwareVersion:
                    description: Software Version.
                    type: str
                  stackNumber:
                    description: Stack Number.
                    type: float
                  state:
                    description: State.
                    type: str
                  sudiSerialNumber:
                    description: Sudi Serial Number.
                    type: str
                type: list
              stackRingProtocol:
                description: Stack Ring Protocol.
                type: str
              supportsStackWorkflows:
                description: Supports Stack Workflows.
                type: bool
              totalMemberCount:
                description: Total Member Count.
                type: float
              validLicenseLevels:
                description: Valid License Levels.
                elements: str
                type: list
            type: dict
          sudiRequired:
            description: Is Sudi Required.
            type: bool
          userMicNumbers:
            description: User Mic Numbers.
            elements: str
            type: list
          userSudiSerialNos:
            description: User Sudi Serial Nos.
            elements: str
            type: list
          workflowId:
            description: Workflow Id.
            type: str
          workflowName:
            description: Workflow Name.
            type: str
        type: dict
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) ImportDevicesInBulkV1
  description: Complete reference of the ImportDevicesInBulkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-devices-in-bulk
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.import_devices_in_bulk_v1,

  - Paths used are
    post /dna/intent/api/v1/onboarding/pnp-device/import,
  - It should be noted that this module is an alias of pnp_device_import_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.pnp_device_import:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - _id: string
      deviceInfo:
        description: string
        deviceSudiSerialNos:
        - string
        hostname: string
        macAddress: string
        pid: string
        serialNumber: string
        siteId: string
        stack: true
        stackInfo:
          isFullRing: true
          stackMemberList:
          - hardwareVersion: string
            licenseLevel: string
            licenseType: string
            macAddress: string
            pid: string
            priority: 0
            role: string
            serialNumber: string
            softwareVersion: string
            stackNumber: 0
            state: string
            sudiSerialNumber: string
          stackRingProtocol: string
          supportsStackWorkflows: true
          totalMemberCount: 0
          validLicenseLevels:
          - string
        sudiRequired: true
        userMicNumbers:
        - string
        userSudiSerialNos:
        - string
        workflowId: string
        workflowName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "successList": [
        {
          "id": "string",
          "deviceInfo": {
            "source": "string",
            "serialNumber": "string",
            "stack": true,
            "mode": "string",
            "state": "string",
            "location": {
              "siteId": "string",
              "address": "string",
              "latitude": "string",
              "longitude": "string",
              "altitude": "string"
            },
            "description": "string",
            "onbState": "string",
            "authenticatedMicNumber": "string",
            "authenticatedSudiSerialNo": "string",
            "capabilitiesSupported": [
              "string"
            ],
            "featuresSupported": [
              "string"
            ],
            "cmState": "string",
            "firstContact": 0,
            "lastContact": 0,
            "macAddress": "string",
            "pid": "string",
            "deviceSudiSerialNos": [
              "string"
            ],
            "lastUpdateOn": 0,
            "workflowId": "string",
            "workflowName": "string",
            "projectId": "string",
            "projectName": "string",
            "deviceType": "string",
            "agentType": "string",
            "imageVersion": "string",
            "fileSystemList": [
              {
                "type": "string",
                "writeable": true,
                "freespace": 0,
                "name": "string",
                "readable": true,
                "size": 0
              }
            ],
            "pnpProfileList": [
              {
                "profileName": "string",
                "discoveryCreated": true,
                "createdBy": "string",
                "primaryEndpoint": {
                  "port": 0,
                  "protocol": "string",
                  "ipv4Address": {},
                  "ipv6Address": {},
                  "fqdn": "string",
                  "certificate": "string"
                },
                "secondaryEndpoint": {
                  "port": 0,
                  "protocol": "string",
                  "ipv4Address": {},
                  "ipv6Address": {},
                  "fqdn": "string",
                  "certificate": "string"
                }
              }
            ],
            "imageFile": "string",
            "httpHeaders": [
              {
                "key": "string",
                "value": "string"
              }
            ],
            "neighborLinks": [
              {
                "localInterfaceName": "string",
                "localShortInterfaceName": "string",
                "localMacAddress": "string",
                "remoteInterfaceName": "string",
                "remoteShortInterfaceName": "string",
                "remoteMacAddress": "string",
                "remoteDeviceName": "string",
                "remotePlatform": "string",
                "remoteVersion": "string"
              }
            ],
            "lastSyncTime": 0,
            "ipInterfaces": [
              {
                "status": "string",
                "macAddress": "string",
                "ipv4Address": {},
                "ipv6AddressList": [
                  {}
                ],
                "name": "string"
              }
            ],
            "hostname": "string",
            "authStatus": "string",
            "stackInfo": {
              "supportsStackWorkflows": true,
              "isFullRing": true,
              "stackMemberList": [
                {
                  "serialNumber": "string",
                  "state": "string",
                  "role": "string",
                  "macAddress": "string",
                  "pid": "string",
                  "licenseLevel": "string",
                  "licenseType": "string",
                  "sudiSerialNumber": "string",
                  "hardwareVersion": "string",
                  "stackNumber": 0,
                  "softwareVersion": "string",
                  "priority": 0
                }
              ],
              "stackRingProtocol": "string",
              "validLicenseLevels": [
                "string"
              ],
              "totalMemberCount": 0
            },
            "reloadRequested": true,
            "addedOn": 0,
            "siteId": "string",
            "aaaCredentials": {
              "password": "string",
              "username": "string"
            },
            "userMicNumbers": [
              "string"
            ],
            "userSudiSerialNos": [
              "string"
            ],
            "addnMacAddrs": [
              "string"
            ],
            "preWorkflowCliOuputs": [
              {
                "cli": "string",
                "cliOutput": "string"
              }
            ],
            "tags": {},
            "sudiRequired": true,
            "smartAccountId": "string",
            "virtualAccountId": "string",
            "populateInventory": true,
            "siteName": "string",
            "name": "string"
          },
          "systemResetWorkflow": {
            "_id": "string",
            "state": "string",
            "type": "string",
            "description": "string",
            "lastupdateOn": 0,
            "imageId": "string",
            "currTaskIdx": 0,
            "addedOn": 0,
            "tasks": [
              {
                "state": "string",
                "type": "string",
                "currWorkItemIdx": 0,
                "taskSeqNo": 0,
                "endTime": 0,
                "startTime": 0,
                "workItemList": [
                  {
                    "state": "string",
                    "command": "string",
                    "outputStr": "string",
                    "endTime": 0,
                    "startTime": 0,
                    "timeTaken": 0
                  }
                ],
                "timeTaken": 0,
                "name": "string"
              }
            ],
            "addToInventory": true,
            "instanceType": "string",
            "endTime": 0,
            "execTime": 0,
            "startTime": 0,
            "useState": "string",
            "configId": "string",
            "name": "string",
            "version": 0,
            "tenantId": "string"
          },
          "systemWorkflow": {
            "_id": "string",
            "state": "string",
            "type": "string",
            "description": "string",
            "lastupdateOn": 0,
            "imageId": "string",
            "currTaskIdx": 0,
            "addedOn": 0,
            "tasks": [
              {
                "state": "string",
                "type": "string",
                "currWorkItemIdx": 0,
                "taskSeqNo": 0,
                "endTime": 0,
                "startTime": 0,
                "workItemList": [
                  {
                    "state": "string",
                    "command": "string",
                    "outputStr": "string",
                    "endTime": 0,
                    "startTime": 0,
                    "timeTaken": 0
                  }
                ],
                "timeTaken": 0,
                "name": "string"
              }
            ],
            "addToInventory": true,
            "instanceType": "string",
            "endTime": 0,
            "execTime": 0,
            "startTime": 0,
            "useState": "string",
            "configId": "string",
            "name": "string",
            "version": 0,
            "tenantId": "string"
          },
          "workflow": {
            "_id": "string",
            "state": "string",
            "type": "string",
            "description": "string",
            "lastupdateOn": 0,
            "imageId": "string",
            "currTaskIdx": 0,
            "addedOn": 0,
            "tasks": [
              {
                "state": "string",
                "type": "string",
                "currWorkItemIdx": 0,
                "taskSeqNo": 0,
                "endTime": 0,
                "startTime": 0,
                "workItemList": [
                  {
                    "state": "string",
                    "command": "string",
                    "outputStr": "string",
                    "endTime": 0,
                    "startTime": 0,
                    "timeTaken": 0
                  }
                ],
                "timeTaken": 0,
                "name": "string"
              }
            ],
            "addToInventory": true,
            "instanceType": "string",
            "endTime": 0,
            "execTime": 0,
            "startTime": 0,
            "useState": "string",
            "configId": "string",
            "name": "string",
            "version": 0,
            "tenantId": "string"
          },
          "runSummaryList": [
            {
              "details": "string",
              "historyTaskInfo": {
                "type": "string",
                "workItemList": [
                  {
                    "state": "string",
                    "command": "string",
                    "outputStr": "string",
                    "endTime": 0,
                    "startTime": 0,
                    "timeTaken": 0
                  }
                ],
                "timeTaken": 0,
                "addnDetails": [
                  {
                    "key": "string",
                    "value": "string"
                  }
                ],
                "name": "string"
              },
              "errorFlag": true,
              "timestamp": 0
            }
          ],
          "workflowParameters": {
            "topOfStackSerialNumber": "string",
            "licenseLevel": "string",
            "licenseType": "string",
            "configList": [
              {
                "configParameters": [
                  {
                    "key": "string",
                    "value": "string"
                  }
                ],
                "configId": "string"
              }
            ]
          },
          "dayZeroConfig": {
            "config": "string"
          },
          "dayZeroConfigPreview": {},
          "version": 0,
          "tenantId": "string"
        }
      ],
      "failureList": [
        {
          "index": 0,
          "serialNum": "string",
          "id": "string",
          "msg": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: ad_hoc_command_wait
author: "John Westcott IV (@john-westcott-iv)"
short_description: Wait for Automation Platform Controller Ad Hoc Command to finish.
description:
    - Wait for Automation Platform Controller ad hoc command to finish and report success or failure. See
      U(https://www.ansible.com/tower) for an overview.
options:
    command_id:
      description:
        - ID of the ad hoc command to monitor.
      required: True
      type: int
    interval:
      description:
        - The interval in sections, to request an update from the controller.
      required: False
      default: 2
      type: float
    timeout:
      description:
        - Maximum time in seconds to wait for a ad hoc command to finish.
      type: int
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Launch an ad hoc command
  ad_hoc_command:
    inventory: "Demo Inventory"
    credential: "Demo Credential"
    wait: false
  register: command

- name: Wait for ad joc command max 120s
  ad_hoc_command_wait:
    command_id: "{{ command.id }}"
    timeout: 120
'''

RETURN = '''
id:
    description: Ad hoc command id that is being waited on
    returned: success
    type: int
    sample: 99
elapsed:
    description: total time in seconds the command took to run
    returned: success
    type: float
    sample: 10.879
started:
    description: timestamp of when the command started running
    returned: success
    type: str
    sample: "2017-03-01T17:03:53.200234Z"
finished:
    description: timestamp of when the command finished running
    returned: success
    type: str
    sample: "2017-03-01T17:04:04.078782Z"
status:
    description: current status of command
    returned: success
    type: str
    sample: successful
'''


from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        command_id=dict(type='int', required=True),
        timeout=dict(type='int'),
        interval=dict(type='float', default=2),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    command_id = module.params.get('command_id')
    timeout = module.params.get('timeout')
    interval = module.params.get('interval')

    # Attempt to look up command based on the provided id
    command = module.get_one(
        'ad_hoc_commands',
        **{
            'data': {
                'id': command_id,
            }
        }
    )

    if command is None:
        module.fail_json(msg='Unable to wait on ad hoc command {0}; that ID does not exist.'.format(command_id))

    # Invoke wait function
    module.wait_on_url(url=command['url'], object_name=command_id, object_type='ad hoc command', timeout=timeout, interval=interval)

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_device_controllability_settings_v1
short_description: Resource module for Network Devices Device Controllability Settings V1
description:
- Manage operation update of the resource Network Devices Device Controllability Settings V1.
- >
   Device Controllability is a system-level process on Catalyst Center that enforces state synchronization for some
   device-layer features. Its purpose is to aid in the deployment of required network settings that Catalyst Center
   needs to manage devices. Changes are made on network devices during discovery, when adding a device to Inventory,
   or when assigning a device to a site. If changes are made to any settings that are under the scope of this
   process, these changes are applied to the network devices during the Provision and Update Telemetry Settings
   operations, even if Device Controllability is disabled. The following device settings will be enabled as part of
   Device Controllability when devices are discovered.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  autocorrectTelemetryConfig:
    description: If it is true, autocorrect telemetry config is enabled. If it is false,
      autocorrect telemetry config is disabled. The autocorrect telemetry config feature
      is supported only when device controllability is enabled.
    type: bool
  deviceControllability:
    description: If it is true, device controllability is enabled. If it is false, device
      controllability is disabled.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design UpdateDeviceControllabilitySettingsV1
  description: Complete reference of the UpdateDeviceControllabilitySettingsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device-controllability-settings
notes:
  - SDK Method used are
    site_design.SiteDesign.update_device_controllability_settings_v1,

  - Paths used are
    put /dna/intent/api/v1/networkDevices/deviceControllability/settings,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.network_devices_device_controllability_settings_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    autocorrectTelemetryConfig: true
    deviceControllability: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



# Code generated by release_script GitHub action; DO NOT EDIT MANUALLY.
ANNOTATION_UNSUPPORTED = [
    "topRoot",
    "monPol",
    "monATarget",
    "monTarget",
    "moTopProps",
    "moModifiable",
    "moOwnable",
    "moResolvable",
    "moASubj",
    "actionACont",
    "taskExec",
    "namingNamedObject",
    "namingNamedIdentifiedObject",
    "conditionInfo",
    "conditionRetP",
    "conditionRecord",
    "conditionLoggable",
    "faultInfo",
    "relnInst",
    "relnTo",
    "relnFrom",
    "statsItem",
    "statsAColl",
    "statsAThrP",
    "relnTaskRef",
    "hvsNode",
    "qosACong",
    "qosAQueue",
    "qosABuffer",
    "qosASched",
    "qosClassification",
    "qosADscpClass",
    "qosADot1PClass",
    "lacpALagPol",
    "pconsRef",
    "eventARetP",
    "faultAThrValue",
    "faultARetP",
    "configABackupP",
    "dnsAProfile",
    "dnsAProv",
    "dnsADomain",
    "dnsALbl",
    "sysdebugRepository",
    "sysdebugFile",
    "sysdebugLogBehavior",
    "firmwareSource",
    "firmwareAFwStatusCont",
    "firmwareARunning",
    "firmwareAFwP",
    "maintAMaintP",
    "maintUserNotif",
    "fabricComp",
    "fabricANode",
    "fabricALink",
    "ruleDefinition",
    "ruleItem",
    "ruleRequirement",
    "ruleSizeRequirement",
    "stpAIfPol",
    "haHaTest",
    "rtctrlAMatchRule",
    "rtctrlAMatchIpRule",
    "rtctrlAMatchRtType",
    "rtctrlASubnet",
    "rtctrlAAttrP",
    "rtctrlASetRule",
    "rtctrlASetTag",
    "rtctrlASetComm",
    "rtctrlASetRtMetric",
    "rtctrlASetPref",
    "rtctrlASetNh",
    "rtctrlASetOspfFwdAddr",
    "rtctrlASetOspfNssa",
    "bgpAAsP",
    "bgpACtxPol",
    "bgpAPeerPfxPol",
    "bgpAPeerP",
    "bgpAExtP",
    "cdpAIfPol",
    "fabricProtoIfPol",
    "fabricL2IfPol",
    "fabricL3IfPol",
    "lldpAIfPol",
    "fabricProtoPol",
    "fabricProtoComp",
    "fabricL2ProtoPol",
    "fabricL3ProtoPol",
    "fabricL2ProtoComp",
    "fabricL3ProtoComp",
    "fabricProtoInstPol",
    "fabricUtilInstPol",
    "fabricL2InstPol",
    "fabricL3InstPol",
    "fabricProtoDomPol",
    "fabricL2DomPol",
    "fabricL3DomPol",
    "fabricL3CtxPol",
    "l2AInstPol",
    "fabricMaintPol",
    "fabricNodeGrp",
    "fabricAPodBlk",
    "fabricANodeBlk",
    "fabricAPortBlk",
    "fabricSelector",
    "fabricANodeS",
    "fabricNodeS",
    "fabricACardS",
    "fabricCardS",
    "fabricAPortS",
    "fabricPortS",
    "fabricIntfPol",
    "fabricAProfile",
    "fabricProfile",
    "fabricPolGrp",
    "fabricNodeP",
    "fabricCardP",
    "fabricPortP",
    "fabricAPortPGrp",
    "fabricANodePGrp",
    "fabricACardPGrp",
    "fabricSpAPortPGrp",
    "fabricLeAPortPGrp",
    "fabricAProtPol",
    "fabricANodePEp",
    "fabricPol",
    "fabricInfrP",
    "fabricInfrExP",
    "fabricDom",
    "fabricDef",
    "fabricAPolGrp",
    "vsvcAProvLbl",
    "vsvcAConsLbl",
    "compASvcVM",
    "compNic",
    "compPNic",
    "compEntity",
    "compElement",
    "compContE",
    "compObj",
    "compCont",
    "compProvP",
    "compDomP",
    "compCtrlrP",
    "compAccessP",
    "compUsrAccP",
    "compHost",
    "compPHost",
    "vzACollection",
    "vzACtrct",
    "vzABrCP",
    "vzAIf",
    "vzAFilterable",
    "vzAFilterableUnit",
    "vzASubj",
    "vzATerm",
    "vzASTerm",
    "vzALbl",
    "vzACompLbl",
    "vzAnyToCollection",
    "vzAFilter",
    "vzAFiltEntry",
    "ospfACtxPol",
    "ospfAIfP",
    "ospfAExtP",
    "dhcpARelayP",
    "dhcpALbl",
    "dhcpAInfraProvP",
    "poolElement",
    "poolPoolable",
    "poolPoolMember",
    "poolPool",
    "pkiItem",
    "aaaDefinition",
    "commDefinition",
    "pkiDefinition",
    "aaaSystemUser",
    "aaaBanner",
    "aaaUserAction",
    "aaaARetP",
    "commComp",
    "commWeb",
    "commShell",
    "aaaRealm",
    "aaaConfig",
    "aaaAuthMethod",
    "aaaEp",
    "aaaAProvider",
    "aaaProviderGroup",
    "adcomATsInfoUnit",
    "adcomARwi",
    "healthARetP",
    "healthAInst",
    "igmpASnoopPol",
    "sysfileEp",
    "sysfileRepository",
    "sysfileInstance",
    "fileARemoteHost",
    "fileARemotePath",
    "monProtoP",
    "monSecAuthP",
    "monGroup",
    "monSubj",
    "monSrc",
    "conditionCondP",
    "ctrlrDom",
    "l2extADomP",
    "l2extALNodeP",
    "l2extAIfP",
    "l2extAInstPSubnet",
    "extnwEPg",
    "extnwOut",
    "extnwDomP",
    "extnwAInstPSubnet",
    "extnwALNodeP",
    "extnwALIfP",
    "l3extADomP",
    "l3extALNodeP",
    "l3extAIfP",
    "l3extAMember",
    "l3extAInstPSubnet",
    "trigSchedWindowP",
    "trigInst",
    "trigWindow",
    "trigSchedWindow",
    "trigExecutable",
    "trigTriggerable",
    "trigSingleTriggerable",
    "fvACont",
    "fvADeplCont",
    "fvL2Dom",
    "fvABD",
    "fvABDPol",
    "fvAEpRetPol",
    "fvComp",
    "fvATg",
    "fvEPgToCollection",
    "fvADomP",
    "fvEPg",
    "fvCEPg",
    "fvAREpPCtrct",
    "fvDom",
    "fvL3Dom",
    "fvACtx",
    "fvNwEp",
    "fvATp",
    "fvEp",
    "fvPEp",
    "fvAEpDef",
    "fvTo",
    "fvFrom",
    "mgmtAZone",
    "mgmtAInstPSubnet",
    "eqptdiagpTestSet",
    "eqptdiagpTestSetBoot",
    "eqptdiagpTestSetHealth",
    "eqptdiagpTestSetOd",
    "eqptdiagpPortTestSetOd",
    "eqptdiagpPortTestSetBt",
    "eqptdiagpPortTestSetHl",
    "eqptdiagpLpTsOd",
    "eqptdiagpFpTsOd",
    "eqptdiagpCardTestSetOd",
    "eqptdiagpSupCTsOd",
    "eqptdiagpSysCTsOd",
    "eqptdiagpFcTsOd",
    "eqptdiagpLcTsOd",
    "eqptdiagpExtChCardTsOd",
    "eqptdiagpPol",
    "eqptdiagpHealthPol",
    "eqptdiagpASynthObj",
    "oamExec",
    "pingAExec",
    "tracerouteAExec",
    "bgpAf",
    "dhcpAddr",
    "dhcpNode",
    "dhcpResp",
    "eqptALPort",
    "dbgACRulePCommon",
    "dbgacTenantSpaceCmn",
    "dbgacEpgCmn",
    "dbgacFromEpgCmn",
    "dbgacToEpgCmn",
    "dbgacFromEpCmn",
    "dbgacToEpCmn",
    "dbgexpExportP",
    "dbgexpNodeStatus",
    "spanASrcGrp",
    "spanASrc",
    "spanASpanLbl",
    "spanADest",
    "spanAVSrc",
    "spanAVSrcGrp",
    "spanAVDestGrp",
    "spanAVDest",
    "svccorePol",
    "svccoreACore",
    "traceroutepTrP",
    "syntheticObject",
    "syntheticAContext",
    "syntheticATestObj",
    "syntheticTLTestObj",
    "syntheticCTestObj",
    "monExportP",
    "ipARouteP",
    "ipANexthopP",
    "infraFexBlk",
    "infraANodeS",
    "infraNodeGrp",
    "infraPortS",
    "infraEPg",
    "infraACEPg",
    "infraAPEPg",
    "infraANode",
    "infraAIpP",
    "infraProfile",
    "infraPolGrp",
    "infraAccBaseGrp",
    "infraFexGrp",
    "infraAccGrp",
    "infraLbl",
    "infraAPEp",
    "infraACEp",
    "infraAFunc",
    "infraGeNode",
    "infraADomP",
    "infraDomP",
    "infraExP",
    "datetimeAPol",
    "datetimeANtpAuthKey",
    "datetimeANtpProv",
    "fvnsAAddrBlk",
    "fvnsAEncapBlk",
    "fvnsAInstP",
    "fvnsAVxlanInstP",
    "fvnsAAddrInstP",
    "polAttTgt",
    "snmpAPol",
    "snmpACommunityP",
    "snmpAUserP",
    "snmpAClientGrpP",
    "snmpAClientP",
    "snmpACtxP",
    "fvDef",
    "fvNp",
    "fvUp",
    "polNToRef",
    "polNFromRef",
    "polObj",
    "polDef",
    "polDefRoot",
    "polComp",
    "polInstr",
    "polCont",
    "polDom",
    "polCtrlr",
    "polCompl",
    "polComplElem",
    "polConsElem",
    "polLbl",
    "polProvLbl",
    "polConsLbl",
    "polIf",
    "polProvIf",
    "polConsIf",
    "polRelnHolder",
    "polNs",
    "polAConfIssues",
    "vnsAGraph",
    "vnsANode",
    "vnsAFuncNode",
    "vnsAFolder",
    "vnsACCfgRel",
    "vnsAParam",
    "vnsATerm",
    "vnsATermNode",
    "vnsAbsTermNode",
    "vnsALDevCtx",
    "vnsALIfCtx",
    "vnsAConn",
    "vnsAFuncConn",
    "vnsATermConn",
    "vnsAConnection",
    "vnsAL4L7ServiceFault",
    "vnsACCfg",
    "vnsALDevIf",
    "vnsALDev",
    "vnsALIf",
    "vnsALDevLIf",
    "vnsDevItem",
    "fabricAPathIssues",
    "conditionSevAsnP",
    "fvAStCEp",
    "fabricNodeToPolicy",
    "aaaADomainRef",
    "fvAEpTaskAggr",
    "vzToRFltP",
    "fvAToBD",
    "frmwrkARelDelCont",
    "infraDomainToNs",
    "infraToAInstP",
    "aaaARbacRule",
    "statsDebugItem",
    "vzInterfaceToCollection",
    "spanAToCEp",
    "fabricAOOSReln",
    "l4AVxlanInstPol",
    "fabricVxlanInstPol",
    "fabricL1IfPol",
    "frmwrkARelDelControl",
    "spanACEpDef",
    "eptrkEpRslt",
    "mcpAIfPol",
    "ipmcsnoopRtrIf",
    "bfdAf",
    "bgpALocalAsnP",
    "bgpACtxAfPol",
    "l3extADefaultRouteLeakP",
    "fvAStIp",
    "rtctrlASetRtMetricType",
    "vnsAEPpInfo",
    "ndAIfPol",
    "ndAPfxPol",
    "eigrpACtxAfPol",
    "eigrpAStubP",
    "eigrpAIfP",
    "eigrpAExtP",
    "l3extAIp",
    "fabricProtoConsFrom",
    "fabricProtoConsTo",
    "fabricAPathS",
    "infraAAccBndlGrp",
    "fabricNodeToPathOverridePolicy",
    "adcomARwiAdvanced",
    "fvAClassifier",
    "fvACrtrn",
    "fvAttr",
    "fvAVmAttr",
    "fvAIpAttr",
    "fvAMacAttr",
    "fvAProtoAttr",
    "polADependentOn",
    "vnsAVRoutingNetworks",
    "l3extARouteTagPol",
    "nwsAFwPol",
    "fabricL4IfPol",
    "fvAVip",
    "fvAAREpPUpdate",
    "conditionSummary",
    "polAPrToPol",
    "polAObjToPolReln",
    "fvACrRule",
    "fvASCrtrn",
    "fvnsAVlanInstP",
    "vzAnyToInterface",
    "fvEPgToInterface",
    "fabricPodGrp",
    "fabricAPodS",
    "infraPodGrp",
    "vnsAMgmt",
    "fabricPolicyGrpToMonitoring",
    "plannerIPs",
    "plannerAObject",
    "plannerAEpg",
    "compAPltfmP",
    "compAVmmPltfmP",
    "compAVmmSecP",
    "bgpARRP",
    "bgpARtTargetP",
    "bgpARtTarget",
    "l3extAFabricExtRoutingP",
    "fvAFabricExtConnP",
    "fvAPodConnP",
    "fvAPeeringP",
    "fvANode",
    "plannerATmpl",
    "nwsASrc",
    "nwsASyslogSrc",
    "throttlerASub",
    "qosADscpTrans",
    "fvARsToRemote",
    "vnsOrchReq",
    "vnsOrchResp",
    "fcprARs",
    "dbgexpTechSupOnDBase",
    "infraGeSnNode",
    "l3extAConsLbl",
    "l3extAProvLbl",
    "trigATriggeredWindow",
    "qosADppPol",
    "bfdAInstPol",
    "bfdAIfPol",
    "bfdAIfP",
    "plannerADomainTmpl",
    "plannerAEpgDomain",
    "vmmACapObj",
    "vmmACapInfo",
    "fvARsToRemoteFC",
    "rtctrlAMatchCommFactor",
    "rtctrlAMatchCommTerm",
    "rtctrlAMatchCommRegexTerm",
    "rtctrlASetDamp",
    "rtctrlASetWeight",
    "hvsContE",
    "hvsUsegContE",
    "compAPvlanP",
    "usegAUsegEPg",
    "fvAStAttr",
    "fvADyAttr",
    "mgmtAIp",
    "faultARsToRemote",
    "analyticsACfgSrv",
    "infraAAccGrp",
    "infraSpAccGrp",
    "infraANodeP",
    "infraPortP",
    "infraAPathS",
    "ipmcsnoopMcSrc",
    "ipmcsnoopTgtIf",
    "analyticsACluster",
    "rtdmcAIfPol",
    "rtdmcARtMapPol",
    "rtdmcAExtP",
    "rtdmcACtxPol",
    "rtdmcARPPol",
    "rtdmcAAutoRPPol",
    "rtdmcABSRPPol",
    "rtdmcAStaticRPPol",
    "rtdmcAStaticRPEntry",
    "rtdmcARPGrpRangePol",
    "rtdmcARegTrPol",
    "rtdmcAResPol",
    "rtdmcAPatPol",
    "rtdmcAASMPatPol",
    "rtdmcASGRangeExpPol",
    "rtdmcASharedRangePol",
    "rtdmcASSMPatPol",
    "rtdmcASSMRangePol",
    "rtdmcABidirPatPol",
    "pimAIfP",
    "rtdmcARtMapEntry",
    "rtdmcAJPFilterPol",
    "rtdmcANbrFilterPol",
    "rtdmcAMAFilter",
    "rtdmcABSRFilter",
    "rtdmcAFilterPol",
    "fvAEPgPathAtt",
    "fvnsAVsanInstP",
    "ipmcAIfPol",
    "ipmcAStRepPol",
    "ipmcAStateLPol",
    "ipmcARepPol",
    "ipmcACtxPol",
    "ipmcASSMXlateP",
    "igmpAIfP",
    "vmmEpgAggr",
    "apPluginName",
    "rtdmcAIfPolCont",
    "fabricL2PortSecurityPol",
    "fcAPinningP",
    "fcAPinningLbl",
    "fabricAONodeS",
    "infraAONodeS",
    "analyticsRemoteNode",
    "analyticsTarget",
    "fabricASubPortBlk",
    "bgpARtTargetInstrP",
    "hsrpAIfPol",
    "hsrpAGroupPol",
    "hsrpAGroupP",
    "hsrpASecVip",
    "hsrpAIfP",
    "netflowARecordPol",
    "netflowAExporterPol",
    "netflowARsInterfaceToMonitor",
    "netflowARsMonitorToRecord",
    "netflowARsMonitorToExporter",
    "netflowAMonitorPol",
    "igmpASnoopStaticGroup",
    "igmpASnoopAccessGroup",
    "netflowAFabExporterPol",
    "fabricQinqIfPol",
    "orchsEntity",
    "orchsElement",
    "vnsNATReq",
    "vnsLBReq",
    "aclACL",
    "aclL3ACE",
    "apDockerName",
    "hostprotASubj",
    "macsecAAIfPol",
    "macsecAIfPol",
    "macsecAAParamPol",
    "macsecAParamPol",
    "macsecAAKeyChainPol",
    "macsecAAKeyPol",
    "macsecAKeyChainPol",
    "macsecAKeyPol",
    "fvAIntersiteConnP",
    "fvAIntersiteConnPDef",
    "fvASiteConnP",
    "coppAProfile",
    "coppACustomValues",
    "infraNodeS",
    "rtctrlASetASPath",
    "rtctrlASetASPathASN",
    "fvADnsAttr",
    "dnsepgFault",
    "dnsepgASvrGrp",
    "dnsepgAMgmt",
    "dnsepgASvr",
    "dnsepgADomain",
    "ipANexthopEpP",
    "vmmInjectedObject",
    "poeAIfPol",
    "iaclAProfile",
    "telemetryAServer",
    "snmpATrapFwdServerP",
    "dwdmAOptChnlIfPol",
    "vzARuleOwner",
    "telemetryAServerPol",
    "telemetryARemoteServer",
    "telemetryAFlowServers",
    "telemetryAServerP",
    "plannerAEpgFilter",
    "callhomeASrc",
    "callhomeADest",
    "callhomeAGroup",
    "fvAEpAnycast",
    "tagTag",
    "tagAnnotation",
    "tagASelector",
    "infraFcAccGrp",
    "infraAFcAccBndlGrp",
    "vsanARsVsanPathAtt",
    "vsanARtVsanPathAtt",
    "vzSubjectToFilter",
    "telemetryAStreamEnable",
    "cloudsecASaKeyP",
    "rtdmcAInterVRFPol",
    "rtdmcAInterVRFEntry",
    "cloudsecAControl",
    "adepgAResElement",
    "adepgAElement",
    "adepgACont",
    "adepgAOrgUnit",
    "adepgEntity",
    "adepgContE",
    "fvAIdAttr",
    "edmObj",
    "edmCont",
    "edmGroupP",
    "edmMgrP",
    "edmEntity",
    "edmElement",
    "edmContE",
    "authASvrGroup",
    "authASvr",
    "authBaseUsrAccP",
    "cloudsecASaKeyPLocal",
    "cloudsecASaKeyPRemote",
    "cloudsecASaKeyStatus",
    "cloudsecASaKeyStatusLocal",
    "cloudsecASaKeyStatusRemote",
    "cloudsecASpKeySt",
    "cloudsecACapability",
    "aaaRbacAnnotation",
    "lacpAEnhancedLagPol",
    "compAHvHealth",
    "mldASnoopPol",
    "fvAExtRoutes",
    "fvAExtRoutableRemoteSitePodSubnet",
    "fvAEpNlb",
    "extdevSDWanASlaPol",
    "eigrpAAuthIfP",
    "fvAAKeyChainPol",
    "fvAAKeyPol",
    "fvAKeyChainPol",
    "fvAKeyPol",
    "fvSyntheticIp",
    "edmACapFlags",
    "cloudABgpPeerP",
    "cloudABgpAsP",
    "cloudARouterP",
    "cloudAIntNetworkP",
    "cloudAExtNetworkP",
    "cloudAHostRouterPol",
    "cloudAHostBootstrapPol",
    "cloudAVpnGwPol",
    "cloudAHostIfP",
    "cloudALoopbackIfP",
    "cloudAL3IfP",
    "cloudAIpv4AddrP",
    "cloudAL3TunnelIfP",
    "cloudAIpSecTunnelIfP",
    "cloudAOspfIfP",
    "cloudAOspfAreaP",
    "cloudACtxProfile",
    "cloudACidr",
    "cloudASubnet",
    "cloudAAwsLogGroup",
    "cloudAAwsFlowLogPol",
    "cloudAProvider",
    "cloudAAwsProvider",
    "cloudAAEPg",
    "cloudAEPSelector",
    "cloudAEPSelectorDef",
    "cloudADomP",
    "genericsARule",
    "ipsecAIsakmpPhase1Pol",
    "ipsecAIsakmpPhase2Pol",
    "resolutionARsToRemote",
    "cloudtemplateASubnetPool",
    "fvAACrtrn",
    "fvAUsegAssocBD",
    "cloudALDev",
    "cloudAListener",
    "cloudAPool",
    "cloudAListenerRule",
    "cloudARuleCondition",
    "cloudARuleAction",
    "cloudACertStore",
    "cloudACertificate",
    "arpAIfPol",
    "compNameIdentEntity",
    "edmAOperCont",
    "edmAStatsCont",
    "cloudASvcPol",
    "hcloudATgStats",
    "statsANWStatsObj",
    "statsATunnel",
    "cloudAController",
    "cloudAApicSubnet",
    "cloudAApicSubnetPool",
    "cloudABaseEPg",
    "cloudASvcEPg",
    "statsAALbStats",
    "mldASnoopStaticGroup",
    "mldASnoopAccessGroup",
    "datetimeANtpIFFKey",
    "hcloudRouterStateOper",
    "vmmAUplinkP",
    "vmmAUplinkPCont",
    "cloudAAFilter",
    "fvASDWanPrefixTaskAggr",
    "cloudAProvResP",
    "bfdAMhInstPol",
    "bfdAMhIfPol",
    "fvAEPSelector",
    "ptpAACfg",
    "ptpACfg",
    "fabricL2BDPol",
    "rtdmcABDPol",
    "rtdmcABDFilter",
    "ptpAProfile",
    "mdpADomP",
    "mdpADomainPeeringPol",
    "mdpAPeeringDomain",
    "mdpANodeP",
    "mdpAClassId",
    "mdpATenant",
    "mplsAExtP",
    "mplsAIfP",
    "mplsALabelPol",
    "mplsANodeSidP",
    "qosAMplsIngressRule",
    "qosMplsMarking",
    "qosAMplsEgressRule",
    "cloudAGatewayRouterP",
    "cloudATransitHubGwPol",
    "cloudAL3IntTunnelIfP",
    "cloudABdiId",
    "mplsASrgbLabelPol",
    "bfdAMhNodePol",
    "bfdANodeP",
    "leakASubnet",
    "rtctrlASetNhUnchanged",
    "mdpAService",
    "cloudALIf",
    "cloudAParamPol",
    "cloudAComputePol",
    "cloudAMgmtPol",
    "cloudANWParams",
    "fvRtScopeFrom",
    "cloudACtxUnderlayP",
    "cloudAHealthProbe",
    "leakARouteCont",
    "netflowAVmmExporterPol",
    "cloudABrownfield",
    "cloudAMapping",
    "cloudASelectedEP",
    "statsANlbStats",
    "mdpAEntity",
    "dbgANodeInst",
    "snmpACtrlrInst",
    "cloudAFrontendIPv4Addr",
    "qosAUburst",
    "telemetryAFteEvents",
    "telemetryAFteEventsTcpFlags",
    "telemetryAFteEventsExt",
    "cloudACloudSvcEPg",
    "hcloudASvcResBase",
    "bgpADomainIdBase",
    "fvAEpTag",
    "bgpASiteOfOriginP",
    "fvAEPSelectorDef",
    "vmmCFaultInfo",
    "synceAAIfPol",
    "synceAIfPol",
    "rtctrlASetRedistMultipath",
    "cloudANextHopIp",
    "cloudAVrfRouteLeakPol",
    "hcloudALeakedPfx",
    "leakAPrefix",
    "fvARogueExceptionMac",
    "bfdAMicroBfdP",
    "hcloudAIntPfx",
    "hcloudAExtPfx",
    "cloudAVpnNetworkP",
    "snmpUser",
    "cloudAVirtualWanP",
    "fvAFBRGroup",
    "fvAFBRoute",
    "fvAFBRMember",
    "rtctrlASetPolicyTag",
    "fvACtxRtSummPol",
    "fvARtSummSubnet",
    "fvAIntraVrfFabricImpRtctrlP",
    "fvAFabricExpRtctrlP",
    "cloudAVip",
    "rtctrlAMatchAsPathRegexTerm",
    "cloudAGcpFlowLogPol",
    "statsAGcpNWStatsObj",
    "cloudABfdPol",
    "cloudABfdP",
    "bgpACtxAddlPathPol",
    "xcvrOpticsIfPol",
    "xcvrOpticsFabIfPol",
    "hostprotANamespace",
    "fvARouteDeployP",
    "rtdmcACSWPol",
    "rtdmcACSWEntry",
    "analyticsTo",
    "analyticsFrom",
    "dmemotltestAbsObject",
    "bgpAsnmpBgpTraps",
    "l3extARogueExceptionMac",
    "l3extARogueExceptionMacGroup",
    "vzAFiltPZEntry",
]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: assurance_issues_top_n_analytics
short_description: Resource module for Assurance Issues Top N Analytics
description:
- This module represents an alias of the module assurance_issues_top_n_analytics_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Assurance Issues Top N Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Assurance Issues Top N Analytics's filters.
    elements: dict
    suboptions:
      filters:
        description: Assurance Issues Top N Analytics's filters.
        elements: dict
        suboptions:
          key:
            description: Key.
            type: str
          operator:
            description: Operator.
            type: str
          value:
            description: Value.
            type: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Assurance Issues Top N Analytics's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Assurance Issues Top N Analytics's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  topN:
    description: Top N.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTopNAnalyticsDataOfIssuesV1
  description: Complete reference of the GetTopNAnalyticsDataOfIssuesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-top-n-analytics-data-of-issues
notes:
  - SDK Method used are
    issues.Issues.get_top_n_analytics_data_of_issues_v1,

  - Paths used are
    post /dna/data/api/v1/assuranceIssues/topNAnalytics,
  - It should be noted that this module is an alias of assurance_issues_top_n_analytics_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assurance_issues_top_n_analytics:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - filters:
      - key: string
        operator: string
        value: string
      key: string
      logicalOperator: string
      operator: string
      value: string
    groupBy:
    - string
    headers: '{{my_headers | from_json}}'
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    topN: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": [
        {
          "id": "string",
          "attributes": [
            {
              "name": "string",
              "value": "string"
            }
          ],
          "aggregateAttributes": [
            {
              "name": "string",
              "function": "string",
              "value": 0
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0,
        "sortBy": [
          {
            "name": "string",
            "function": {},
            "order": "string"
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_tenant_span_dst_group
short_description: Manage SPAN destination groups (span:DestGrp)
description:
- Manage SPAN destination groups on Cisco ACI fabrics.
options:
  destination_group:
    description:
    - The name of the SPAN destination group.
    type: str
    aliases: [ name, dst_group ]
  description:
    description:
    - The description of the SPAN destination group.
    type: str
    aliases: [ descr ]
  tenant:
    description:
    - The name of the tenant.
    type: str
    aliases: [ tenant_name ]
  destination_epg:
    description:
    - The destination end point group.
    type: dict
    suboptions:
      epg:
        description:
        - The name of the end point group.
        type: str
      ap:
        description:
        - The name of application profile.
        type: str
      tenant:
        description:
        - The name of the tenant.
        type: str
        aliases: [ tenant_name ]
  source_ip:
    description:
    - The source IP address or prefix.
    type: str
  destination_ip:
    description:
    - The destination IP address.
    type: str
  span_version:
    description:
    - SPAN version.
    type: str
    choices: [ version_1, version_2 ]
  flow_id:
    description:
    - The flow ID of the SPAN packet.
    type: int
  ttl:
    description:
    - The time to live of the span session packets.
    type: int
  mtu:
    description:
    - The MTU truncation size for the packets.
    type: int
  dscp:
    description:
    - The DSCP value for sending the monitored packets using ERSPAN.
    type: str
    choices: [ CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, unspecified ]
  version_enforced:
    description:
    - Enforce SPAN version.
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(span:DestGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add SPAN destination group
  cisco.aci.aci_tenant_span_dst_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Tenant1
    destination_epg:
      tenant: Test1
      ap: ap1
      epg: ep1
    destination_group: group1
    description: Test span
    destination_ip: 10.0.0.1
    source_ip: 10.0.2.1
    version_enforced: false
    span_version: version_1
    ttl: 2
    mtu: 1500
    flow_id: 1
    dscp: CS1
    state: present
  delegate_to: localhost

- name: Remove SPAN destination group
  cisco.aci.aci_tenant_span_dst_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Tenant1
    destination_group: group1
    state: absent
  delegate_to: localhost

- name: Query SPAN destination group
  cisco.aci.aci_tenant_span_dst_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Tenant1
    destination_group: group1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all SPAN destination groups
  cisco.aci.aci_tenant_span_dst_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def destination_epg_spec():
    return dict(
        epg=dict(type="str"),
        ap=dict(type="str"),
        tenant=dict(type="str", aliases=["tenant_name"]),
    )


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        destination_epg=dict(type="dict", options=destination_epg_spec()),
        destination_group=dict(type="str", aliases=["name", "dst_group"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        name_alias=dict(type="str"),
        source_ip=dict(type="str"),
        destination_ip=dict(type="str"),
        mtu=dict(type="int"),
        ttl=dict(type="int"),
        flow_id=dict(type="int"),
        version_enforced=dict(type="bool"),
        span_version=dict(type="str", choices=["version_1", "version_2"]),
        dscp=dict(
            type="str",
            choices=[
                "CS0",
                "CS1",
                "CS2",
                "CS3",
                "CS4",
                "CS5",
                "CS6",
                "CS7",
                "EF",
                "VA",
                "AF11",
                "AF12",
                "AF13",
                "AF21",
                "AF22",
                "AF23",
                "AF31",
                "AF32",
                "AF33",
                "AF41",
                "AF42",
                "AF43",
                "unspecified",
            ],
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["destination_group", "tenant"]],
            ["state", "present", ["destination_group", "destination_ip", "source_ip", "destination_epg", "tenant"]],
        ],
    )

    aci = ACIModule(module)

    destination_epg = module.params.get("destination_epg")
    destination_group = module.params.get("destination_group")
    description = module.params.get("description")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    destination_ip = module.params.get("destination_ip")
    source_ip = module.params.get("source_ip")
    span_version = module.params.get("span_version")
    name_alias = module.params.get("name_alias")
    dscp = module.params.get("dscp")
    mtu = str(module.params.get("mtu"))
    ttl = str(module.params.get("ttl"))
    flow_id = str(module.params.get("flow_id"))
    version_enforced = module.params.get("version_enforced")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="spanDestGrp",
            aci_rn="destgrp-{0}".format(destination_group),
            module_object=destination_group,
            target_filter={"name": destination_group},
        ),
        child_classes=["spanDest", "spanRsDestEpg"],
    )

    aci.get_existing()

    if state == "present":
        dest_tdn = "uni/tn-{0}/ap-{1}/epg-{2}".format(destination_epg["tenant"], destination_epg["ap"], destination_epg["epg"])

        if version_enforced is True:
            version_enforced = "yes"
        else:
            version_enforced = "no"

        if span_version == "version_1":
            span_version = "ver1"
        else:
            span_version = "ver2"

        child_configs = [
            dict(
                spanDest=dict(
                    attributes=dict(name=destination_group),
                    children=[
                        dict(
                            spanRsDestEpg=dict(
                                attributes=dict(
                                    ip=destination_ip,
                                    srcIpPrefix=source_ip,
                                    ver=span_version,
                                    verEnforced=version_enforced,
                                    ttl=ttl,
                                    mtu=mtu,
                                    flowId=flow_id,
                                    dscp=dscp,
                                    tDn=dest_tdn,
                                )
                            )
                        )
                    ],
                )
            ),
        ]

        aci.payload(
            aci_class="spanDestGrp",
            class_config=dict(
                name=destination_group,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="spanDestGrp")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Logging_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def tmplt_host(config_data):
    command = "logging"
    if config_data.get("host"):
        command += " {host}".format(host=config_data["host"])
    if config_data.get("vrf"):
        command += " vrf {vrf}".format(vrf=config_data["vrf"])
    if config_data.get("severity"):
        command += " severity {severity}".format(
            severity=config_data["severity"],
        )
    if config_data.get("port"):
        command += " port {port}".format(port=config_data["port"])
    return command


def tmplt_correlator_rule(config_data):
    commands = []
    rule_name = config_data.get("rule_name")
    rule_type = config_data.get("rule_type")
    base_command = "logging correlator rule {rule_name} type {rule_type}".format(
        rule_type=rule_type,
        rule_name=rule_name,
    )
    if config_data.get("timeout"):
        commands.append(
            "{base_command} timeout {timeout}".format(
                timeout=config_data["timeout"],
                base_command=base_command,
            ),
        )
    if config_data.get("timeout_rootcause"):
        commands.append(
            "{base_command} timeout-rootcause {timeout}".format(
                timeout=config_data["timeout_rootcause"],
                base_command=base_command,
            ),
        )
    if config_data.get("reissue_nonbistate"):
        commands.append(
            "{base_command} reissue-nonbistate".format(
                base_command=base_command,
            ),
        )
    if config_data.get("reparent"):
        commands.append(
            "{base_command} reparent".format(base_command=base_command),
        )
    if config_data.get("context_correlation"):
        commands.append(
            "{base_command} context-correlation".format(
                base_command=base_command,
            ),
        )
    return commands


def tmplt_correlator_ruleset(config_data):
    command = "logging correlator ruleset"
    if config_data.get("name"):
        command += "  {name}".format(name=config_data["name"])
    if config_data.get("rulename"):
        command += "  rulename {rulename}".format(
            rulename=config_data["rulename"],
        )
    return command


def tmplt_files(config_data):
    command = "logging"
    if config_data.get("name"):
        command += " file {name}".format(name=config_data["name"])
    if config_data.get("path"):
        command += " path {path}".format(path=config_data["path"])
    if config_data.get("maxfilesize"):
        command += " maxfilesize {maxfilesize}".format(
            maxfilesize=config_data["maxfilesize"],
        )
    if config_data.get("severity"):
        command += " severity {severity}".format(
            severity=config_data["severity"],
        )
    return command


def tmplt_source_interface(config_data):
    command = "logging source-interface"
    if config_data.get("interface"):
        command += " {interface}".format(interface=config_data["interface"])
    if config_data.get("vrf"):
        command += " vrf {vrf}".format(vrf=config_data["vrf"])
    return command


def tmplt_tls_servers(config_data):
    commands = []
    name = config_data.get("name")
    base_command = "logging tls-server {name}".format(name=name)
    if config_data.get("tls_hostname"):
        commands.append(
            "{base_command} tls-hostname {tls}".format(
                tls=config_data["tls_hostname"],
                base_command=base_command,
            ),
        )
    if config_data.get("trustpoint"):
        commands.append(
            "{base_command} trustpoint {trustpoint}".format(
                trustpoint=config_data["trustpoint"],
                base_command=base_command,
            ),
        )
    if config_data.get("vrf"):
        commands.append(
            "{base_command} vrf {vrf}".format(
                vrf=config_data["vrf"],
                base_command=base_command,
            ),
        )
    if config_data.get("severity"):
        commands.append(
            "{base_command} severity {severity}".format(
                severity=config_data["severity"],
                base_command=base_command,
            ),
        )
    if len(commands) == 0:
        commands.append(base_command)
    return commands


def rem_tmplt_tls_servers(config_data):
    command = "logging tls-server"
    if config_data.get("name"):
        command += " {name}".format(name=config_data["name"])
    return command


class Logging_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Logging_globalTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "archive.device",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+device\s(?P<device>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging archive device {{archive.device}}",
            "result": {
                "archive": {
                    "device": "{{ device }}",
                },
            },
        },
        {
            "name": "archive.severity",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+severity\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warnings))
                $""", re.VERBOSE,
            ),
            "setval": "logging archive severity {{archive.severity}}",
            "result": {
                "archive": {
                    "severity": "{{severity}}",
                },
            },
        },
        {
            "name": "archive.file_size",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+file-size\s(?P<file_size>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging archive file-size {{archive.file_size}}",
            "result": {
                "archive": {
                    "file_size": "{{ file_size }}",
                },
            },
        },
        {
            "name": "archive.frequency",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+frequency\s(?P<frequency>daily|weekly))?
                $""", re.VERBOSE,
            ),
            "setval": "logging archive frequency {{archive.frequency}}",
            "result": {
                "archive": {
                    "frequency": "{{ frequency }}",
                },
            },
        },
        {
            "name": "archive.archive_size",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+archive-size\s(?P<archive_size>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging archive archive-size {{archive.archive_size}}",
            "result": {
                "archive": {
                    "archive_size": "{{ archive_size }}",
                },
            },
        },
        {
            "name": "archive.archive_length",
            "getval": re.compile(
                r"""
                ^logging\sarchive
                (\s+archive-length\s(?P<archive_length>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging archive archive-length {{archive.archive_length}}",
            "result": {
                "archive": {
                    "archive_length": "{{ archive_length}}",
                },
            },
        },
        {
            "name": "hosts",
            "getval": re.compile(
                r"""
                ^logging
                (\s(?P<host>\S+))?
                (\svrf\s(?P<vrf>\w+))?
                (\sseverity\s(?P<severity>alerts|critical|debugging|emergencies|error|informational|notifications|warnings))?
                (\sport\s(?P<port>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_host,
            "result": {
                "hosts": [{
                    "host": "{{ host }}",
                    "port": "{{ port }}",
                    "vrf": "{{ vrf }}",
                    "severity": "{{severity}}",
                }],
            },

        },
        {
            "name": "buffered.size",
            "getval": re.compile(
                r"""
                ^logging\sbuffered
                (\s(?P<size>[1-9][0-9]*))?
                $""", re.VERBOSE,
            ),
            "setval": "logging buffered {{buffered.size}}",
            "result": {
                "buffered": {
                    "size": "{{ size }}",
                },

            },
        },
        {
            "name": "buffered.severity",
            "getval": re.compile(
                r"""
                ^logging\sbuffered
                (\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warnings))?
                $""", re.VERBOSE,
            ),
            "setval": "logging buffered {{buffered.severity}}",
            "result": {

                "buffered": {
                    "severity": "{{ severity }}",
                },
            },

        },
        {
            "name": "buffered.discriminator",
            "getval": re.compile(
                r"""
                ^logging\sbuffered\sdiscriminator
                \s+(?P<match_nomatch>\S+)\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "logging buffered discriminator {{match_params}} {{name}}",
            "result": {
                "buffered": {
                    "discriminator": [
                        {
                            "match_params": "{{ match_nomatch }}",
                            "name": "{{name}}",
                        },
                    ],
                },

            },
        },
        {
            "name": "console.severity",
            "getval": re.compile(
                r"""
                ^logging\sconsole
                (\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warning$))?
                $""", re.VERBOSE,
            ),
            "setval": "logging console {{console.severity}}",
            "result": {
                "console": {
                    "severity": "{{ severity }}",
                },
            },
        },
        {
            "name": "console.state",
            "getval": re.compile(
                r"""
                ^logging\sconsole
                \s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'logging console disable' if console.state =='disabled' else 'no logging console disable' }}",
            "result": {
                "console": {
                    "state": "{{ 'disabled' if disable is defined }}",
                },
            },
        },
        {
            "name": "console.discriminator",
            "getval": re.compile(
                r"""
                ^logging\sconsole\sdiscriminator
                \s+(?P<match_nomatch>\S+)\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "logging console discriminator {{match_params}} {{name}}",
            "result": {
                "console": {
                    "discriminator": [
                        {
                            "match_params": "{{ match_nomatch }}",
                            "name": "{{name}}",
                        },
                    ],
                },
            },
        },
        {
            "name": "correlator.buffer_size",
            "getval": re.compile(
                r"""
                ^logging\scorrelator
                (\sbuffer-size\s(?P<buffer_size>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging correlator buffer-size {{correlator.buffer_size }}",
            "result": {
                "correlator": {
                    "buffer_size": "{{ buffer_size }}",
                },
            },
        },
        {
            "name": "correlator.rules",
            "getval": re.compile(
                r"""
                ^logging\scorrelator
                (\srule\s(?P<name>\S+))?
                (\stype\s(?P<rule_type>\S+))?
                (\s+reissue-nonbistate(?P<reissue_nonbistate>))?
                (\s+timeout\s(?P<timeout>\S+))?
                (\s+reparent(?P<reparent>))?
                (\s+context-correlation(?P<context_correlation>))?
                (\s+timeout-rootcause\s(?P<timeout_rootcause>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_correlator_rule,
            "result": {
                "correlator": {
                    "rules": {
                        "{{name}}": {
                            "rule_name": "{{ name }}",
                            "rule_type": "{{ rule_type }}",
                            "reissue_nonbistate": "{{ True if reissue_nonbistate is defined }}",
                            "timeout": "{{ timeout }}",
                            "reparent": "{{ True if reparent is defined }}",
                            "context_correlation": "{{ True if context_correlation is defined }}",
                            "timeout_rootcause": "{{ timeout_rootcause }}",

                        },
                    },
                },
            },
        },
        {
            "name": "correlator.rule_sets",
            "getval": re.compile(
                r"""
                ^logging\scorrelator\sruleset\s(?P<name>\S+)
                (\srulename\s(?P<rulename>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_correlator_ruleset,
            "result": {
                "correlator": {
                    "rule_sets": {
                        "{{name}}": {
                            "name": "{{ name }}",
                            "rulename": ["{{ rulename }}"],

                        },
                    },
                },
            },
        },
        {
            "name": "events.threshold",
            "getval": re.compile(
                r"""
                ^logging\sevents
                (\sthreshold\s(?P<threshold>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging events threshold {{events.threshold}}",
            "result": {
                "events":
                    {"threshold": "{{ threshold }}"},
            },

        },
        {
            "name": "events.buffer_size",
            "getval": re.compile(
                r"""
                ^logging\sevents
                (\sbuffer-size\s(?P<buffer_size>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "logging events buffer-size {{events.buffer_size}}",
            "result": {
                "events": {"buffer_size": "{{ buffer_size }}"},
            },

        },
        {
            "name": "events.display_location",
            "getval": re.compile(
                r"""
                ^logging\sevents
                (\sdisplay-location(?P<display_location>))?
                $""", re.VERBOSE,
            ),
            "setval": "logging events display-location",
            "result": {
                "events":
                    {
                        "display_location": "{{ True if display_location is defined }}",

                    },

            },
        },
        {
            "name": "events.filter_match",
            "getval": re.compile(
                r"""
                ^logging\sevents\sfilter
                (\s+match\s(?P<match>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging events filter match {{match}}",
            "result": {
                "events": {
                    "filter_match": [
                        "{{ match }}",
                    ],
                },
            },
        },
        {
            "name": "events.severity",
            "getval": re.compile(
                r"""
                ^logging\sevents
                (\slevel\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warnings))?
                $""", re.VERBOSE,
            ),
            "setval": "logging events level {{events.severity}}",
            "result": {
                "events":
                    {
                        "severity": "{{ severity }}",

                    },
            },
        },
        {
            "name": "files",
            "getval": re.compile(
                r"""
                ^logging
                (\sfile\s(?P<file>\S+))?
                (\spath\s(?P<path>\S+))?
                (\smaxfilesize\s(?P<maxfilesize>\S+))?
                (\sseverity\s(?P<severity>alerts|critical|debugging|emergencies|errors|info|notifications|warning))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_files,
            "result": {
                "files": [
                    {
                        "name": "{{file}}",
                        "path": "{{path}}",
                        "maxfilesize": "{{maxfilesize}}",
                        "severity": "{{ severity  }}",

                    },
                ],
            },
        },
        {
            "name": "facility",
            "getval": re.compile(
                r"""
                ^logging\sfacility
                \s(?P<facility>auth|cron|daemon|kern|local0|local1|local2|local3|local4|local5|local6|local7|lpr|mail|news|sys10|sys11|sys12|sys13|sys14|sys9|syslog|user|uucp)
                $""", re.VERBOSE,
            ),
            "setval": "logging facility {{ facility }}",
            "result": {
                "facility": "{{ facility }}",
            },
        },
        {
            "name": "format",
            "getval": re.compile(
                r"""
                ^logging\sformat
                (\srfc5424(?P<format>))
                $""", re.VERBOSE,
            ),
            "setval": "logging format rfc5424",
            "result": {
                "format": "{{True if format is defined}}",
            },
        },
        {
            "name": "hostnameprefix",
            "getval": re.compile(
                r"""
                ^logging
                (\shostnameprefix\s(?P<hostnameprefix>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging hostnameprefix {{hostnameprefix}}",
            "result": {
                "hostnameprefix": "{{hostnameprefix}}",
            },
        },
        {
            "name": "ipv4.dscp",
            "getval": re.compile(
                r"""
                ^logging
                \sipv4\sdscp\s(?P<dscp>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "logging ipv4 dscp {{ipv4.dscp}}",
            "result": {
                "ipv4": {
                    "dscp": "{{dscp}}",
                },
            },
        },
        {
            "name": "ipv6.dscp",
            "getval": re.compile(
                r"""
                ^logging
                (\sipv6\sdscp\s(?P<dscp>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging ipv6 dscp {{ipv6.dscp}}",
            "result": {
                "ipv6": {
                    "dscp": "{{dscp}}",
                },
            },
        },
        {
            "name": "ipv4.precedence",
            "getval": re.compile(
                r"""
                ^logging
                (\sipv4\sprecedence\s(?P<precedence>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging ipv4 precedence {{ipv4.precedence}}",
            "result": {
                "ipv4": {"precedence": "{{precedence}}"},
            },
        },
        {
            "name": "ipv6.precedence",
            "getval": re.compile(
                r"""
                ^logging
                (\sipv6\sprecedence\s(?P<precedence>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging ipv6 precedence {{ipv6.precedence}}",
            "result": {
                "ipv6": {"precedence": "{{precedence}}"},
            },
        },
        {
            "name": "localfilesize",
            "getval": re.compile(
                r"""
                ^logging
                (\slocalfilesize\s(?P<localfilesize>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "logging localfilesize {{localfilesize}}",
            "result": {
                "localfilesize": "{{localfilesize}}",
            },
        },
        {
            "name": "suppress.duplicates",
            "getval": re.compile(
                r"""
                ^logging
                (\ssuppress\sduplicates(?P<suppress_duplicates>))
                $""", re.VERBOSE,
            ),
            "setval": "logging suppress duplicates",
            "result": {
                "suppress":
                    {"duplicates": "{{True if suppress_duplicates is defined}}"},
            },
        },
        {
            "name": "suppress.apply_rule",
            "getval": re.compile(
                r"""
                ^logging\ssuppress\sapply\srule\s(?P<apply_rules>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "logging suppress apply rule {{suppress.apply_rule}}",
            "result": {
                "suppress": {"apply_rule": "{{apply_rules}}"},
            },
        },
        {
            "name": "history.size",
            "getval": re.compile(
                r"""
                ^logging\shistory
                (\ssize\s(?P<size>\d+))
                $""", re.VERBOSE,
            ),
            "setval": "logging history size {{history.size}}",
            "result": {
                "history": {
                    "severity": "{{ severity }}",
                    "size": "{{ size }}",
                },
            },
        },
        {
            "name": "history.severity",
            "getval": re.compile(
                r"""
                ^logging\shistory
                (\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warning))
                $""", re.VERBOSE,
            ),
            "setval": "logging history {{history.severity}}",
            "result": {
                "history": {
                    "severity": "{{ severity }}",
                },
            },
        },
        {
            "name": "history.state",
            "getval": re.compile(
                r"""
                ^logging\shistory
                \s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'logging history disable' if history.state =='disabled' else 'no logging history disable' }}",
            "result": {
                "history": {
                    "state": "{{ 'disabled' if disable is defined }}",
                },
            },
        },
        {
            "name": "monitor.severity",
            "getval": re.compile(
                r"""
                ^logging\smonitor
                (\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warning))?
                $""", re.VERBOSE,
            ),
            "setval": "logging monitor {{monitor.severity}}",
            "result": {
                "monitor": {
                    "severity": "{{ severity }}",
                },
            },
        },
        {
            "name": "monitor.state",
            "getval": re.compile(
                r"""
                ^logging\smonitor\s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'logging monitor disable' if monitor.state =='disabled' else 'no logging monitor disable' }}",
            "result": {
                "monitor": {
                    "state": "{{ 'disabled' if disable is defined }}",
                },
            },
        },
        {
            "name": "monitor.discriminator",
            "getval": re.compile(
                r"""
                ^logging\smonitor\sdiscriminator
                \s+(?P<match_nomatch>\S+)\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "logging monitor discriminator {{match_params}} {{name}}",
            "result": {
                "monitor": {
                    "discriminator": [
                        {
                            "match_params": "{{ match_nomatch }}",
                            "name": "{{name}}",
                        },
                    ],
                },

            },
        },
        {
            "name": "source_interfaces",
            "getval": re.compile(
                r"""
                ^logging\ssource-interface
                (\s(?P<interface>\S+))?
                (\svrf\s(?P<vrf>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_source_interface,
            "result": {
                "source_interfaces": [
                    {
                        "interface": "{{ interface }}",
                        "vrf": "{{ vrf }}",
                    },
                ],
            },
        },
        {
            "name": "trap.severity",
            "getval": re.compile(
                r"""
                ^logging\strap
                \s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warning)
                $""", re.VERBOSE,
            ),
            "setval": "logging trap {{ trap.severity }}",
            "result": {
                "trap": {
                    "severity": "{{ severity }}",
                },
            },
        },
        {
            "name": "trap.state",
            "getval": re.compile(
                r"""
                ^logging\strap
                \s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "{{ 'logging trap disable' if trap.state =='disabled' else 'no logging trap disable' }}",
            "result": {
                "trap": {
                    "state": "{{ 'disabled' if disable is defined }}",
                },
            },
        },
        {
            "name": "tls_servers",
            "getval": re.compile(
                r"""
                ^logging\stls-server\s(?P<name>\S+)
                (\s+vrf\s(?P<vrf>\S+))?
                (\s+trustpoint\s(?P<trustpoint>\S+))?
                (\s+tls-hostname\s(?P<tls_hostname>\S+))?
                (\s(?P<severity>alerts|critical|debugging|emergencies|errors|informational|notifications|warnings))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_tls_servers,
            "remval": rem_tmplt_tls_servers,
            "result": {
                "tls_servers": {
                    "{{name}}": {
                        "name": "{{ name }}",
                        "trustpoint": "{{trustpoint}}",
                        "vrf": "{{vrf}}",
                        "severity": "{{severity}}",
                        "tls_hostname": "{{tls_hostname}}",
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: floors_upload_image_v2
short_description: Resource module for Floors Upload Image V2
description:
- Manage operation create of the resource Floors Upload Image V2.
- Uploads floor image.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Floor Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design UploadsFloorImageV2
  description: Complete reference of the UploadsFloorImageV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!uploads-floor-image-v-2
notes:
  - SDK Method used are
    site_design.SiteDesign.uploads_floor_image_v2,

  - Paths used are
    post /dna/intent/api/v2/floors/{id}/uploadImage,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.floors_upload_image_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: diagnostic_validation_workflows
short_description: Resource module for Diagnostic Validation Workflows
description:
- This module represents an alias of the module diagnostic_validation_workflows_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Description of the workflow to run.
    type: str
  id:
    description: Id path parameter. Workflow id.
    type: str
  name:
    description: Name of the workflow to run. It must be unique.
    type: str
  validationSetIds:
    description: List of validation set ids.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Health and Performance SubmitsTheWorkflowForExecutingValidationsV1
  description: Complete reference of the SubmitsTheWorkflowForExecutingValidationsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!submits-the-workflow-for-executing-validations
- name: Cisco DNA Center documentation for Health and Performance DeletesAValidationWorkflowV1
  description: Complete reference of the DeletesAValidationWorkflowV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-a-validation-workflow
notes:
  - SDK Method used are
    health_and_performance.HealthAndPerformance.deletes_a_validation_workflow_v1,
    health_and_performance.HealthAndPerformance.submits_the_workflow_for_executing_validations_v1,

  - Paths used are
    post /dna/intent/api/v1/diagnosticValidationWorkflows,
    delete /dna/intent/api/v1/diagnosticValidationWorkflows/{id},
  - It should be noted that this module is an alias of diagnostic_validation_workflows_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.diagnostic_validation_workflows:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    name: string
    validationSetIds:
    - string

- name: Delete by id
  cisco.dnac.diagnostic_validation_workflows:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: cloudtrail
version_added: 5.0.0
short_description: manage CloudTrail create, delete, update
description:
  - Creates, deletes, or updates CloudTrail configuration. Ensures logging is also enabled.
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - Ansible Core Team
  - Ted Timmons (@tedder)
  - Daniel Shepherd (@shepdelacreme)
options:
  state:
    description:
      - Add or remove CloudTrail configuration.
      - 'The following states have been preserved for backwards compatibility: O(state=enabled) and O(state=disabled).'
      - O(state=enabled) is equivalet to O(state=present).
      - O(state=disabled) is equivalet to O(state=absent).
    type: str
    choices: ['present', 'absent', 'enabled', 'disabled']
    default: present
  name:
    description:
      - Name for the CloudTrail.
      - Names are unique per-region unless the CloudTrail is a multi-region trail, in which case it is unique per-account.
    type: str
    default: default
  enable_logging:
    description:
      - Start or stop the CloudTrail logging. If stopped the trail will be paused and will not record events or deliver log files.
    default: true
    type: bool
  s3_bucket_name:
    description:
      - An existing S3 bucket where CloudTrail will deliver log files.
      - This bucket should exist and have the proper policy.
      - See U(https://docs.aws.amazon.com/awscloudtrail/latest/userguide/aggregating_logs_regions_bucket_policy.html).
      - Required when O(state=present).
    type: str
  s3_key_prefix:
    description:
      - S3 Key prefix for delivered log files. A trailing slash is not necessary and will be removed.
    type: str
  is_multi_region_trail:
    description:
      - Specify whether the trail belongs only to one region or exists in all regions.
    default: false
    type: bool
  enable_log_file_validation:
    description:
      - Specifies whether log file integrity validation is enabled.
      - CloudTrail will create a hash for every log file delivered and produce a signed digest file that can be used to ensure log files have not been tampered.
    type: bool
    aliases: [ "log_file_validation_enabled" ]
  include_global_events:
    description:
      - Record API calls from global services such as IAM and STS.
    default: true
    type: bool
    aliases: [ "include_global_service_events" ]
  sns_topic_name:
    description:
      - SNS Topic name to send notifications to when a log file is delivered.
    type: str
  cloudwatch_logs_role_arn:
    description:
      - Specifies a full ARN for an IAM role that assigns the proper permissions for CloudTrail to create and write to the log group.
      - See U(https://docs.aws.amazon.com/awscloudtrail/latest/userguide/send-cloudtrail-events-to-cloudwatch-logs.html).
      - Required when O(cloudwatch_logs_log_group_arn).
    type: str
  cloudwatch_logs_log_group_arn:
    description:
      - A full ARN specifying a valid CloudWatch log group to which CloudTrail logs will be delivered. The log group should already exist.
      - See U(https://docs.aws.amazon.com/awscloudtrail/latest/userguide/send-cloudtrail-events-to-cloudwatch-logs.html).
      - Required when O(cloudwatch_logs_role_arn).
    type: str
  kms_key_id:
    description:
      - Specifies the KMS key ID to use to encrypt the logs delivered by CloudTrail. This also has the effect of enabling log file encryption.
      - The value can be an alias name prefixed by "alias/", a fully specified ARN to an alias, a fully specified ARN to a key, or a globally unique identifier.
      - Encryption can be disabled by setting O(kms_key_id="").
      - See U(https://docs.aws.amazon.com/awscloudtrail/latest/userguide/encrypting-cloudtrail-log-files-with-aws-kms.html).
    type: str
notes:
  - The O(purge_tags) option was added in release 4.0.0.

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: create single region cloudtrail
  amazon.aws.cloudtrail:
    state: present
    name: default
    s3_bucket_name: mylogbucket
    s3_key_prefix: cloudtrail
    region: us-east-1

- name: create multi-region trail with validation and tags
  amazon.aws.cloudtrail:
    state: present
    name: default
    s3_bucket_name: mylogbucket
    region: us-east-1
    is_multi_region_trail: true
    enable_log_file_validation: true
    cloudwatch_logs_role_arn: "arn:aws:iam::123456789012:role/CloudTrail_CloudWatchLogs_Role"
    cloudwatch_logs_log_group_arn: "arn:aws:logs:us-east-1:123456789012:log-group:CloudTrail/DefaultLogGroup:*"
    kms_key_id: "alias/MyAliasName"
    tags:
      environment: dev
      Name: default

- name: show another valid kms_key_id
  amazon.aws.cloudtrail:
    state: present
    name: default
    s3_bucket_name: mylogbucket
    kms_key_id: "arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012"
    # simply "12345678-1234-1234-1234-123456789012" would be valid too.

- name: pause logging the trail we just created
  amazon.aws.cloudtrail:
    state: present
    name: default
    enable_logging: false
    s3_bucket_name: mylogbucket
    region: us-east-1
    is_multi_region_trail: true
    enable_log_file_validation: true
    tags:
      environment: dev
      Name: default

- name: delete a trail
  amazon.aws.cloudtrail:
    state: absent
    name: default
"""

RETURN = r"""
exists:
    description: whether the resource exists.
    returned: always
    type: bool
    sample: true
trail:
    description: CloudTrail resource details.
    returned: always
    type: complex
    sample: hash/dictionary of values
    contains:
        trail_arn:
            description: Full ARN of the CloudTrail resource.
            returned: success
            type: str
            sample: arn:aws:cloudtrail:us-east-1:123456789012:trail/default
        name:
            description: Name of the CloudTrail resource.
            returned: success
            type: str
            sample: default
        is_logging:
            description: Whether logging is turned on or paused for the Trail.
            returned: success
            type: bool
            sample: True
        s3_bucket_name:
            description: S3 bucket name where log files are delivered.
            returned: success
            type: str
            sample: myBucket
        s3_key_prefix:
            description: Key prefix in bucket where log files are delivered (if any).
            returned: success when present
            type: str
            sample: myKeyPrefix
        log_file_validation_enabled:
            description: Whether log file validation is enabled on the trail.
            returned: success
            type: bool
            sample: true
        include_global_service_events:
            description: Whether global services (IAM, STS) are logged with this trail.
            returned: success
            type: bool
            sample: true
        is_multi_region_trail:
            description: Whether the trail applies to all regions or just one.
            returned: success
            type: bool
            sample: true
        is_organization_trail:
            description:
            - Specifies whether the trail is created for all accounts in an organization in Organizations, or only for the current Amazon Web Services account.
            returned: success
            type: bool
            sample: true
        has_custom_event_selectors:
            description: Whether any custom event selectors are used for this trail.
            returned: success
            type: bool
            sample: False
        has_insight_selectors:
            description: Whether any insight selectors are used for this trail.
            returned: success
            type: bool
            sample: False
        home_region:
            description: The home region where the trail was originally created and must be edited.
            returned: success
            type: str
            sample: us-east-1
        sns_topic_name:
            description: The SNS topic name where log delivery notifications are sent.
            returned: success when present
            type: str
            sample: myTopic
        sns_topic_arn:
            description: Full ARN of the SNS topic where log delivery notifications are sent.
            returned: success when present
            type: str
            sample: arn:aws:sns:us-east-1:123456789012:topic/myTopic
        cloud_watch_logs_log_group_arn:
            description: Full ARN of the CloudWatch Logs log group where events are delivered.
            returned: success when present
            type: str
            sample: arn:aws:logs:us-east-1:123456789012:log-group:CloudTrail/DefaultLogGroup:*
        cloud_watch_logs_role_arn:
            description: Full ARN of the IAM role that CloudTrail assumes to deliver events.
            returned: success when present
            type: str
            sample: arn:aws:iam::123456789012:role/CloudTrail_CloudWatchLogs_Role
        kms_key_id:
            description: Full ARN of the KMS Key used to encrypt log files.
            returned: success when present
            type: str
            sample: arn:aws:kms::123456789012:key/12345678-1234-1234-1234-123456789012
        tags:
            description: hash/dictionary of tags applied to this resource
            returned: success
            type: dict
            sample: {'environment': 'dev', 'Name': 'default'}
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


def get_kms_key_aliases(module, client, keyId):
    """
    get list of key aliases

    module : AnsibleAWSModule object
    client : boto3 client connection object for kms
    keyId : keyId to get aliases for
    """
    try:
        key_resp = client.list_aliases(KeyId=keyId)
    except (BotoCoreError, ClientError):
        # Don't fail here, just return [] to maintain backwards compat
        # in case user doesn't have kms:ListAliases permissions
        return []

    return key_resp["Aliases"]


def create_trail(module, client, ct_params):
    """
    Creates a CloudTrail

    module : AnsibleAWSModule object
    client : boto3 client connection object
    ct_params : The parameters for the Trail to create
    """
    resp = {}
    try:
        resp = client.create_trail(**ct_params)
    except (BotoCoreError, ClientError) as err:
        module.fail_json_aws(err, msg="Failed to create Trail")

    return resp


def tag_trail(module, client, tags, trail_arn, curr_tags=None, purge_tags=True):
    """
    Creates, updates, removes tags on a CloudTrail resource

    module : AnsibleAWSModule object
    client : boto3 client connection object
    tags : Dict of tags converted from ansible_dict to boto3 list of dicts
    trail_arn : The ARN of the CloudTrail to operate on
    curr_tags : Dict of the current tags on resource, if any
    dry_run : true/false to determine if changes will be made if needed
    """

    if tags is None:
        return False

    curr_tags = curr_tags or {}

    tags_to_add, tags_to_remove = compare_aws_tags(curr_tags, tags, purge_tags=purge_tags)
    if not tags_to_add and not tags_to_remove:
        return False

    if module.check_mode:
        return True

    if tags_to_remove:
        remove = {k: curr_tags[k] for k in tags_to_remove}
        tags_to_remove = ansible_dict_to_boto3_tag_list(remove)
        try:
            client.remove_tags(ResourceId=trail_arn, TagsList=tags_to_remove)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to remove tags from Trail")

    if tags_to_add:
        tags_to_add = ansible_dict_to_boto3_tag_list(tags_to_add)
        try:
            client.add_tags(ResourceId=trail_arn, TagsList=tags_to_add)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to add tags to Trail")

    return True


def set_logging(module, client, name, action):
    """
    Starts or stops logging based on given state

    module : AnsibleAWSModule object
    client : boto3 client connection object
    name : The name or ARN of the CloudTrail to operate on
    action : start or stop
    """
    if action == "start":
        try:
            client.start_logging(Name=name)
            return client.get_trail_status(Name=name)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to start logging")
    elif action == "stop":
        try:
            client.stop_logging(Name=name)
            return client.get_trail_status(Name=name)
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to stop logging")
    else:
        module.fail_json(msg="Unsupported logging action")


def get_trail_facts(module, client, name):
    """
    Describes existing trail in an account

    module : AnsibleAWSModule object
    client : boto3 client connection object
    name : Name of the trail
    """
    # get Trail info
    try:
        trail_resp = client.describe_trails(trailNameList=[name])
    except (BotoCoreError, ClientError) as err:
        module.fail_json_aws(err, msg="Failed to describe Trail")

    # Now check to see if our trail exists and get status and tags
    if len(trail_resp["trailList"]):
        trail = trail_resp["trailList"][0]
        try:
            status_resp = client.get_trail_status(Name=trail["Name"])
            tags_list = client.list_tags(ResourceIdList=[trail["TrailARN"]])
        except (BotoCoreError, ClientError) as err:
            module.fail_json_aws(err, msg="Failed to describe Trail")

        trail["IsLogging"] = status_resp["IsLogging"]
        trail["tags"] = boto3_tag_list_to_ansible_dict(tags_list["ResourceTagList"][0]["TagsList"])
        # Check for non-existent values and populate with None
        optional_vals = set(
            [
                "S3KeyPrefix",
                "SnsTopicName",
                "SnsTopicARN",
                "CloudWatchLogsLogGroupArn",
                "CloudWatchLogsRoleArn",
                "KmsKeyId",
            ]
        )
        for v in optional_vals - set(trail.keys()):
            trail[v] = None
        return trail

    else:
        # trail doesn't exist return None
        return None


def delete_trail(module, client, trail_arn):
    """
    Delete a CloudTrail

    module : AnsibleAWSModule object
    client : boto3 client connection object
    trail_arn : Full CloudTrail ARN
    """
    try:
        client.delete_trail(Name=trail_arn)
    except (BotoCoreError, ClientError) as err:
        module.fail_json_aws(err, msg="Failed to delete Trail")


def update_trail(module, client, ct_params):
    """
    Delete a CloudTrail

    module : AnsibleAWSModule object
    client : boto3 client connection object
    ct_params : The parameters for the Trail to update
    """
    try:
        client.update_trail(**ct_params)
    except (BotoCoreError, ClientError) as err:
        module.fail_json_aws(err, msg="Failed to update Trail")


def main():
    argument_spec = dict(
        state=dict(default="present", choices=["present", "absent", "enabled", "disabled"]),
        name=dict(default="default"),
        enable_logging=dict(default=True, type="bool"),
        s3_bucket_name=dict(),
        s3_key_prefix=dict(no_log=False),
        sns_topic_name=dict(),
        is_multi_region_trail=dict(default=False, type="bool"),
        enable_log_file_validation=dict(type="bool", aliases=["log_file_validation_enabled"]),
        include_global_events=dict(default=True, type="bool", aliases=["include_global_service_events"]),
        cloudwatch_logs_role_arn=dict(),
        cloudwatch_logs_log_group_arn=dict(),
        kms_key_id=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
    )

    required_if = [("state", "present", ["s3_bucket_name"]), ("state", "enabled", ["s3_bucket_name"])]
    required_together = [("cloudwatch_logs_role_arn", "cloudwatch_logs_log_group_arn")]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_together=required_together,
        required_if=required_if,
    )

    # collect parameters
    if module.params["state"] in ("present", "enabled"):
        state = "present"
    elif module.params["state"] in ("absent", "disabled"):
        state = "absent"
    tags = module.params["tags"]
    purge_tags = module.params["purge_tags"]
    enable_logging = module.params["enable_logging"]
    ct_params = dict(
        Name=module.params["name"],
        S3BucketName=module.params["s3_bucket_name"],
        IncludeGlobalServiceEvents=module.params["include_global_events"],
        IsMultiRegionTrail=module.params["is_multi_region_trail"],
    )

    if module.params["s3_key_prefix"]:
        ct_params["S3KeyPrefix"] = module.params["s3_key_prefix"].rstrip("/")

    if module.params["sns_topic_name"]:
        ct_params["SnsTopicName"] = module.params["sns_topic_name"]

    if module.params["cloudwatch_logs_role_arn"]:
        ct_params["CloudWatchLogsRoleArn"] = module.params["cloudwatch_logs_role_arn"]

    if module.params["cloudwatch_logs_log_group_arn"]:
        ct_params["CloudWatchLogsLogGroupArn"] = module.params["cloudwatch_logs_log_group_arn"]

    if module.params["enable_log_file_validation"] is not None:
        ct_params["EnableLogFileValidation"] = module.params["enable_log_file_validation"]

    if module.params["kms_key_id"] is not None:
        ct_params["KmsKeyId"] = module.params["kms_key_id"]

    client = module.client("cloudtrail")
    region = module.region

    results = dict(changed=False, exists=False)

    # Get existing trail facts
    trail = get_trail_facts(module, client, ct_params["Name"])

    # If the trail exists set the result exists variable
    if trail is not None:
        results["exists"] = True
        initial_kms_key_id = trail.get("KmsKeyId")

    if state == "absent" and results["exists"]:
        # If Trail exists go ahead and delete
        results["changed"] = True
        results["exists"] = False
        results["trail"] = dict()
        if not module.check_mode:
            delete_trail(module, client, trail["TrailARN"])

    elif state == "present" and results["exists"]:
        # If Trail exists see if we need to update it
        do_update = False
        for key in ct_params:
            tkey = str(key)
            # boto3 has inconsistent parameter naming so we handle it here
            if key == "EnableLogFileValidation":
                tkey = "LogFileValidationEnabled"
            # We need to make an empty string equal None
            if ct_params.get(key) == "":
                val = None
            else:
                val = ct_params.get(key)
            if val != trail.get(tkey):
                do_update = True
                if tkey != "KmsKeyId":
                    # We'll check if the KmsKeyId casues changes later since
                    # user could've provided a key alias, alias arn, or key id
                    # and trail['KmsKeyId'] is always a key arn
                    results["changed"] = True
                # If we are in check mode copy the changed values to the trail facts in result output to show what would change.
                if module.check_mode:
                    trail.update({tkey: ct_params.get(key)})

        if not module.check_mode and do_update:
            update_trail(module, client, ct_params)
            trail = get_trail_facts(module, client, ct_params["Name"])

        # Determine if KmsKeyId changed
        if not module.check_mode:
            if initial_kms_key_id != trail.get("KmsKeyId"):
                results["changed"] = True
        else:
            new_key = ct_params.get("KmsKeyId")
            if initial_kms_key_id != new_key:
                # Assume changed for a moment
                results["changed"] = True

                # However, new_key could be a key id, alias arn, or alias name
                # that maps back to the key arn in initial_kms_key_id. So check
                # all aliases for a match.
                initial_aliases = get_kms_key_aliases(module, module.client("kms"), initial_kms_key_id)
                for a in initial_aliases:
                    if a["AliasName"] == new_key or a["AliasArn"] == new_key or a["TargetKeyId"] == new_key:
                        results["changed"] = False

        # Check if we need to start/stop logging
        if enable_logging and not trail["IsLogging"]:
            results["changed"] = True
            trail["IsLogging"] = True
            if not module.check_mode:
                set_logging(module, client, name=ct_params["Name"], action="start")
        if not enable_logging and trail["IsLogging"]:
            results["changed"] = True
            trail["IsLogging"] = False
            if not module.check_mode:
                set_logging(module, client, name=ct_params["Name"], action="stop")

        # Check if we need to update tags on resource
        tags_changed = tag_trail(
            module, client, tags=tags, trail_arn=trail["TrailARN"], curr_tags=trail["tags"], purge_tags=purge_tags
        )
        if tags_changed:
            updated_tags = dict()
            if not purge_tags:
                updated_tags = trail["tags"]
            updated_tags.update(tags)
            results["changed"] = True
            trail["tags"] = updated_tags

        # Populate trail facts in output
        results["trail"] = camel_dict_to_snake_dict(trail, ignore_list=["tags"])

    elif state == "present" and not results["exists"]:
        # Trail doesn't exist just go create it
        results["changed"] = True
        results["exists"] = True
        if not module.check_mode:
            if tags:
                ct_params["TagsList"] = ansible_dict_to_boto3_tag_list(tags)
            # If we aren't in check_mode then actually create it
            created_trail = create_trail(module, client, ct_params)
            # Get the trail status
            try:
                status_resp = client.get_trail_status(Name=created_trail["Name"])
            except (BotoCoreError, ClientError) as err:
                module.fail_json_aws(err, msg="Failed to fetch Trail statuc")
            # Set the logging state for the trail to desired value
            if enable_logging and not status_resp["IsLogging"]:
                set_logging(module, client, name=ct_params["Name"], action="start")
            if not enable_logging and status_resp["IsLogging"]:
                set_logging(module, client, name=ct_params["Name"], action="stop")
            # Get facts for newly created Trail
            trail = get_trail_facts(module, client, ct_params["Name"])

        # If we are in check mode create a fake return structure for the newly minted trail
        if module.check_mode:
            acct_id = "123456789012"
            try:
                sts_client = module.client("sts")
                acct_id = sts_client.get_caller_identity()["Account"]
            except (BotoCoreError, ClientError):
                pass
            trail = dict()
            trail.update(ct_params)
            if "EnableLogFileValidation" not in ct_params:
                ct_params["EnableLogFileValidation"] = False
            trail["EnableLogFileValidation"] = ct_params["EnableLogFileValidation"]
            trail.pop("EnableLogFileValidation")
            fake_arn = "arn:aws:cloudtrail:" + region + ":" + acct_id + ":trail/" + ct_params["Name"]
            trail["HasCustomEventSelectors"] = False
            trail["HomeRegion"] = region
            trail["TrailARN"] = fake_arn
            trail["IsLogging"] = enable_logging
            trail["tags"] = tags
        # Populate trail facts in output
        results["trail"] = camel_dict_to_snake_dict(trail, ignore_list=["tags"])

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: integration_settings_itsm_instances_info
short_description: Information module for Integration Settings Itsm Instances Info
description:
- This module represents an alias of the module integration_settings_itsm_instances_v1_info
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for ITSM Integration GetAllITSMIntegrationSettingsV1
  description: Complete reference of the GetAllITSMIntegrationSettingsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-all-itsm-integration-settings
notes:
  - SDK Method used are
    itsm_integration.ItsmIntegration.get_all_itsm_integration_settings_v1,

  - Paths used are
    get /dna/intent/api/v1/integration-settings/itsm/instances,
  - It should be noted that this module is an alias of integration_settings_itsm_instances_v1_info

"""

EXAMPLES = r"""
- name: Get all Integration Settings Itsm Instances Info
  cisco.dnac.integration_settings_itsm_instances_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "dypId": "string",
        "dypName": "string",
        "name": "string",
        "uniqueKey": "string",
        "dypMajorVersion": 0,
        "description": "string",
        "createdDate": 0,
        "createdBy": "string",
        "updatedBy": "string",
        "softwareVersionLog": [
          {}
        ],
        "schemaVersion": 0,
        "tenantId": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: interfaces_query_count
short_description: Resource module for Interfaces Query Count
description:
- This module represents an alias of the module interfaces_query_count_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Interfaces Query Count's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Interfaces Query Count's filters.
    elements: dict
    suboptions:
      filters:
        description: Filters.
        elements: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: dict
    type: list
  page:
    description: Interfaces Query Count's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Interfaces Query Count's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  views:
    description: Views.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices TheTotalInterfacesCountAcrossTheNetworkDevicesV1
  description: Complete reference of the TheTotalInterfacesCountAcrossTheNetworkDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!the-total-interfaces-count-across-the-network-devices
notes:
  - SDK Method used are
    devices.Devices.the_total_interfaces_count_across_the_network_devices_v1,

  - Paths used are
    post /dna/data/api/v1/interfaces/query/count,
  - It should be noted that this module is an alias of interfaces_query_count_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.interfaces_query_count:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - filters:
      - string
      key: string
      logicalOperator: string
      operator: string
      value: {}
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    views:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tags_network_devices_members_associations_v1_info
short_description: Information module for Tags Network Devices Members Associations V1
description:
- Get all Tags Network Devices Members Associations V1.
- >
   Fetches the tags associated with network devices. Devices that don't have any tags associated will not be included
   in the response. A tag is a user-defined or system-defined construct to group resources. When a device is tagged,
   it is called a member of the tag.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter. The first record to show for this page; the first record is numbered 1. Minimum 1.
    type: float
  limit:
    description:
    - Limit query parameter. The number of records to show for this page. Minimum 1, maximum 500.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Tag RetrieveTagsAssociatedWithNetworkDevicesV1
  description: Complete reference of the RetrieveTagsAssociatedWithNetworkDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieve-tags-associated-with-network-devices
notes:
  - SDK Method used are
    tag.Tag.retrieve_tags_associated_with_network_devices_v1,

  - Paths used are
    get /dna/intent/api/v1/tags/networkDevices/membersAssociations,

"""

EXAMPLES = r"""
- name: Get all Tags Network Devices Members Associations V1
  cisco.dnac.tags_network_devices_members_associations_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "tags": [
            {
              "id": "string",
              "name": "string"
            }
          ]
        }
      ],
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr lldp fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.lldp_global.lldp_global import (
    Lldp_globalArgs,
)


class Lldp_globalFacts(object):
    """The iosxr lldp fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Lldp_globalArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        return connection.get_config(flags="lldp")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lldp
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_config(connection)

        obj = {}
        if "lldp" in data:
            lldp_obj = self.render_config(self.generated_spec, data)
            if lldp_obj:
                obj = lldp_obj

        ansible_facts["ansible_network_resources"].pop("lldp_global", None)
        facts = {}

        params = utils.validate_config(self.argument_spec, {"config": obj})
        facts["lldp_global"] = utils.remove_empties(params["config"])

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        for key in spec.keys():
            if key == "subinterfaces":
                config[key] = True if "subinterfaces enable" in conf else None

            elif key == "tlv_select":
                for item in [
                    "system_name",
                    "port_description",
                    "management_address",
                    "system_description",
                    "system_capabilities",
                ]:
                    config[key][item] = (
                        False if ("{0} disable".format(item.replace("_", "-"))) in conf else None
                    )

            else:
                value = utils.parse_conf_arg(conf, key)
                config[key] = int(value) if value else value

        return config



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: buildings
short_description: Resource module for Buildings
description:
- This module represents an alias of the module buildings_v2
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  address:
    description: Building address. Example 4900 Marie P. Debartolo Way, Santa Clara,
      California 95054, United States.
    type: str
  country:
    description: Country name.
    type: str
  id:
    description: Id path parameter. Building Id.
    type: str
  latitude:
    description: Building Latitude. Example 37.403712.
    type: float
  longitude:
    description: Building Longitude. Example -121.971063.
    type: float
  name:
    description: Building name.
    type: str
  parentId:
    description: Parent Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design CreatesABuildingV2
  description: Complete reference of the CreatesABuildingV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!creates-a-building-v-2
- name: Cisco DNA Center documentation for Site Design DeletesABuildingV2
  description: Complete reference of the DeletesABuildingV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-a-building-v-2
- name: Cisco DNA Center documentation for Site Design UpdatesABuildingV2
  description: Complete reference of the UpdatesABuildingV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!updates-a-building-v-2
notes:
  - SDK Method used are
    site_design.SiteDesign.creates_a_building_v2,
    site_design.SiteDesign.deletes_a_building_v2,
    site_design.SiteDesign.updates_a_building_v2,

  - Paths used are
    post /dna/intent/api/v2/buildings,
    delete /dna/intent/api/v2/buildings/{id},
    put /dna/intent/api/v2/buildings/{id},
  - It should be noted that this module is an alias of buildings_v2

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.buildings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    address: string
    country: string
    latitude: 0
    longitude: 0
    name: string
    parentId: string

- name: Update by id
  cisco.dnac.buildings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    address: string
    country: string
    id: string
    latitude: 0
    longitude: 0
    name: string
    parentId: string

- name: Delete by id
  cisco.dnac.buildings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The parse_cli filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: parse_cli
author: Peter Sprygada (@privateip)
version_added: "1.0.0"
short_description: parse_cli filter plugin.
description:
  - The filter plugins converts the output of a network device
    CLI command into structured JSON output.
  - Using the parameters below - C(xml_data | ansible.netcommon.parse_cli(template.yml))
notes:
  - The parse_cli filter will load the spec file and pass the command output through it,
    returning JSON output. The YAML spec file defines how to parse the CLI output
options:
  output:
    description:
    - This source data on which parse_cli invokes.
    type: raw
    required: True
  tmpl:
    description:
    - The spec file should be valid formatted YAML.
      It defines how to parse the CLI output and return JSON data.
    - For example C(cli_data | ansible.netcommon.parse_cli(template.yml)),
      in this case C(cli_data) represents cli output.
    type: str
"""

EXAMPLES = r"""
# Using parse_cli

# outputConfig

# ip dhcp pool Data
#   import all
#   network 192.168.1.0 255.255.255.0
#   update dns
#   default-router 192.168.1.1
#   dns-server 192.168.1.1 8.8.8.8
#   option 42 ip 192.168.1.1
#   domain-name test.local
#   lease 8

# pconnection.yml

# ---
# vars:
#   dhcp_pool:
#     name: "{{ item.name }}"
#     network: "{{ item.network_ip }}"
#     subnet: "{{ item.network_subnet }}"
#     dns_servers: "{{ item.dns_servers_1 }}{{ item.dns_servers_2 }}"
#     domain_name: "{{ item.domain_name_0 }}{{ item.domain_name_1 }}{{ item.domain_name_2 }}{{ item.domain_name_3 }}"
#     options: "{{ item.options_1 }}{{ item.options_2 }}"
#     lease_days: "{{ item.lease_days }}"
#     lease_hours: "{{ item.lease_hours }}"
#     lease_minutes: "{{ item.lease_minutes }}"

# keys:
#   dhcp_pools:
#     value: "{{ dhcp_pool }}"
#     items: "^ip dhcp pool (
#           ?P<name>[^\\n]+)\\s+(?:import (?P<import_all>all)\\s*)?(?:network (?P<network_ip>[\\d.]+)
#           (?P<network_subnet>[\\d.]+)?\\s*)?(?:update dns\\s*)?(?:host (?P<host_ip>[\\d.]+)
#           (?P<host_subnet>[\\d.]+)\\s*)?(?:domain-name (?P<domain_name_0>[\\w._-]+)\\s+)?
#           (?:default-router (?P<default_router>[\\d.]+)\\s*)?(?:dns-server
#           (?P<dns_servers_1>(?:[\\d.]+ ?)+ ?)+\\s*)?(?:domain-name (?P<domain_name_1>[\\w._-]+)\\s+)?
#           (?P<options_1>(?:option [^\\n]+\\n*\\s*)*)?(?:domain-name (?P<domain_name_2>[\\w._-]+)\\s+)?(?P<options_2>(?:option [^\\n]+\\n*\\s*)*)?
#           (?:dns-server (?P<dns_servers_2>(?:[\\d.]+ ?)+ ?)+\\s*)?(?:domain-name
#           (?P<domain_name_3>[\\w._-]+)\\s*)?(lease (?P<lease_days>\\d+)(?: (?P<lease_hours>\\d+))?(?: (?P<lease_minutes>\\d+))?\\s*)?(?:update arp)?"

# playbook

- name: Add config data
  ansible.builtin.set_fact:
    opconfig: "{{lookup('ansible.builtin.file', 'outputConfig') }}"

- name: Parse Data
  ansible.builtin.set_fact:
    output: "{{ opconfig | parse_cli('pconnection.yml') }}"


# Task Output
# -----------
#
# TASK [Add config data]
# ok: [host] => changed=false
#   ansible_facts:
#     xml: |-
#       ip dhcp pool Data
#         import all
#         network 192.168.1.0 255.255.255.0
#         update dns
#         default-router 192.168.1.1
#         dns-server 192.168.1.1 8.8.8.8
#         option 42 ip 192.168.1.1
#         domain-name test.local
#         lease 8

# TASK [Parse Data]
# ok: [host] => changed=false
#   ansible_facts:
#     output:
#       dhcp_pools:
#       - dns_servers: 192.168.1.1 8.8.8.8
#         domain_name: test.local
#         lease_days: 8
#         lease_hours: null
#         lease_minutes: null
#         name: Data
#         network: 192.168.1.0
#         options: |-
#           option 42 ip 192.168.1.1
#         subnet: 255.255.255.0
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.parse_cli import parse_cli


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _parse_cli(*args, **kwargs):
    """Extend vlan data"""

    keys = ["output", "tmpl"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="parse_cli")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return parse_cli(**updated_data)


class FilterModule(object):
    """parse_cli"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"parse_cli": _parse_cli}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: planned_access_points_v1
short_description: Resource module for Planned Access Points V1
description:
- Manage operations create, update and delete of the resource Planned Access Points V1.
- >
   Allows creation of a new planned access point on an existing floor map including its planned radio and antenna
   details. Use the Get variant of this API to fetch any existing planned access points for the floor. The payload to
   create a planned access point is in the same format, albeit a single object instead of a list, of that API.
- >
   Allow to delete a planned access point from an existing floor map including its planned radio and antenna details.
   Use the Get variant of this API to fetch the existing planned access points for the floor. The instanceUUID listed
   in each of the planned access point attributes acts as the path param input to this API to delete that specific
   instance.
- >
   Allows updating a planned access point on an existing floor map including its planned radio and antenna details.
   Use the Get variant of this API to fetch the existing planned access points for the floor. The payload to update a
   planned access point is in the same format, albeit a single object instead of a list, of that API.
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  attributes:
    description: Planned Access Points's attributes.
    suboptions:
      createDate:
        description: Created date of the planned access point.
        type: float
      domain:
        description: Service domain to which the planned access point belongs.
        type: str
      heirarchyName:
        description: Hierarchy name of the planned access point.
        type: str
      id:
        description: Unique id of the planned access point.
        type: float
      instanceUuid:
        description: Instance uuid of the planned access point.
        type: str
      macAddress:
        description: MAC address of the planned access point.
        type: str
      name:
        description: Display name of the planned access point.
        type: str
      source:
        description: Source of the data used to create the planned access point.
        type: str
      typeString:
        description: Type string representation of the planned access point.
        type: str
    type: dict
  floorId:
    description: FloorId path parameter. The instance UUID of the floor hierarchy element.
    type: str
  isSensor:
    description: Indicates that PAP is a sensor.
    type: bool
  location:
    description: Planned Access Points's location.
    suboptions:
      altitude:
        description: Altitude of the planned access point's location.
        type: float
      lattitude:
        description: Latitude of the planned access point's location.
        type: float
      longtitude:
        description: Longitude of the planned access point's location.
        type: float
    type: dict
  plannedAccessPointUuid:
    description: PlannedAccessPointUuid path parameter. The instance UUID of the planned
      access point to delete.
    type: str
  position:
    description: Planned Access Points's position.
    suboptions:
      x:
        description: X-coordinate of the planned access point on the map, 0,0 point
          being the top-left corner.
        type: float
      y:
        description: Y-coordinate of the planned access point on the map, 0,0 point
          being the top-left corner.
        type: float
      z:
        description: Z-coordinate, or height, of the planned access point on the map.
        type: float
    type: dict
  radioCount:
    description: Number of radios of the planned access point.
    type: int
  radios:
    description: Planned Access Points's radios.
    elements: dict
    suboptions:
      antenna:
        description: Planned Access Points's antenna.
        suboptions:
          azimuthAngle:
            description: Azimuth angle of the antenna.
            type: float
          elevationAngle:
            description: Elevation angle of the antenna.
            type: float
          gain:
            description: Gain of the antenna.
            type: float
          mode:
            description: Mode of the antenna associated with this radio.
            type: str
          name:
            description: Name of the antenna.
            type: str
          type:
            description: Type of the antenna associated with this radio.
            type: str
        type: dict
      attributes:
        description: Planned Access Points's attributes.
        suboptions:
          channel:
            description: Channel in which this radio operates.
            type: float
          channelString:
            description: Channel string representation.
            type: str
          id:
            description: Id of the radio.
            type: int
          ifMode:
            description: IF mode of the radio.
            type: str
          ifTypeString:
            description: String representation of native band.
            type: str
          ifTypeSubband:
            description: Sub band of the radio.
            type: str
          instanceUuid:
            description: Instance Uuid of the radio.
            type: str
          slotId:
            description: Slot number in which the radio resides in the parent access
              point.
            type: float
          txPowerLevel:
            description: Tx Power at which this radio operates (in dBm).
            type: float
        type: dict
      isSensor:
        description: Determines if it is sensor or not.
        type: bool
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices CreatePlannedAccessPointForFloorV1
  description: Complete reference of the CreatePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-planned-access-point-for-floor
- name: Cisco DNA Center documentation for Devices DeletePlannedAccessPointForFloorV1
  description: Complete reference of the DeletePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-planned-access-point-for-floor
- name: Cisco DNA Center documentation for Devices UpdatePlannedAccessPointForFloorV1
  description: Complete reference of the UpdatePlannedAccessPointForFloorV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-planned-access-point-for-floor
notes:
  - SDK Method used are
    devices.Devices.create_planned_access_point_for_floor_v1,
    devices.Devices.delete_planned_access_point_for_floor_v1,
    devices.Devices.update_planned_access_point_for_floor_v1,

  - Paths used are
    post /dna/intent/api/v1/floors/{floorId}/planned-access-points,
    delete /dna/intent/api/v1/floors/{floorId}/planned-access-points/{plannedAccessPointUuid},
    put /dna/intent/api/v1/floors/{floorId}/planned-access-points,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.planned_access_points_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    attributes:
      createDate: 0
      domain: string
      heirarchyName: string
      id: 0
      instanceUuid: string
      macAddress: string
      name: string
      source: string
      typeString: string
    floorId: string
    isSensor: true
    location:
      altitude: 0
      lattitude: 0
      longtitude: 0
    position:
      x: 0
      y: 0
      z: 0
    radioCount: 0
    radios:
    - antenna:
        azimuthAngle: 0
        elevationAngle: 0
        gain: 0
        mode: string
        name: string
        type: string
      attributes:
        channel: 0
        channelString: string
        id: 0
        ifMode: string
        ifTypeString: string
        ifTypeSubband: string
        instanceUuid: string
        slotId: 0
        txPowerLevel: 0
      isSensor: true

- name: Create
  cisco.dnac.planned_access_points_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    attributes:
      createDate: 0
      domain: string
      heirarchyName: string
      id: 0
      instanceUuid: string
      macAddress: string
      name: string
      source: string
      typeString: string
    floorId: string
    isSensor: true
    location:
      altitude: 0
      lattitude: 0
      longtitude: 0
    position:
      x: 0
      y: 0
      z: 0
    radioCount: 0
    radios:
    - antenna:
        azimuthAngle: 0
        elevationAngle: 0
        gain: 0
        mode: string
        name: string
        type: string
      attributes:
        channel: 0
        channelString: string
        id: 0
        ifMode: string
        ifTypeString: string
        ifTypeSubband: string
        instanceUuid: string
        slotId: 0
        txPowerLevel: 0
      isSensor: true

- name: Delete by id
  cisco.dnac.planned_access_points_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    floorId: string
    plannedAccessPointUuid: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""
